﻿using System;
using System.Collections.Generic;
using System.Windows.Threading;
using Telerik.Windows.Controls;
using System.Windows.Documents;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
#if WPF
using System.IO;
#endif

namespace Telerik.Windows.QuickStart
{
#if !WPF
	using FlowDocument = TextBlock;
	using FlowDocumentScrollViewer = ScrollViewer;
	using System.IO;
#endif

	public class CodeViewer : BaseCodeViewer
	{
		private Dictionary<string, string> sourceCode = new Dictionary<string, string>();
		internal RadTabControl TabControl;

		public CodeViewer()
		{
			this.DefaultStyleKey = typeof(CodeViewer);
		}

		public override void OnApplyTemplate()
		{
			this.InitializeTabControl();
			this.InitializeFlowDocument();
		}

		private void InitializeFlowDocument()
		{
			this.FormattedTextScrollViewer = this.GetTemplateChild("ScrollViewer") as FlowDocumentScrollViewer;
			this.FormattedText = this.GetTemplateChild("FormattedText") as FlowDocument;
#if !WPF
			this.Selection = (TextBox)this.GetTemplateChild("Selection");
			this.Selection.KeyDown += (sender, e) => { e.Handled = true; };
#endif
		}

		private void InitializeTabControl()
		{
			if (this.TabControl != null)
			{
				this.TabControl.SelectionChanged -= this.OnTabControlSelectionChanged;
			}
			this.TabControl = this.GetTemplateChild("TabControl") as RadTabControl;
			if (this.TabControl != null)
			{
				this.TabControl.SelectionChanged += this.OnTabControlSelectionChanged;
			}
		}

		private void OnTabControlSelectionChanged(object sender, RoutedEventArgs e)
		{
			this.DisplaySelectedFile();
		}

		private void DisplaySelectedFile()
		{
			string selectedFile = this.TabControl.SelectedItem as string;
			if (selectedFile == null)
			{
				return;
			}

			InlineCollection inlines = this.ResetCodeViewerContent();

			if (this.sourceCode.ContainsKey(System.IO.Path.GetFileName(selectedFile)))
			{
				string code = this.sourceCode[selectedFile];

				Dispatcher.BeginInvoke(DispatcherPriority.ApplicationIdle,
					new Action(() =>
					{
						inlines = this.ResetCodeViewerContent();
						FormatCodeFile(inlines, code, GetSourceFileExtension(selectedFile));

						RefreshTextArea();
					}));
			}
			inlines.Add(new Run() { Text = "Loading...." });

			RefreshTextArea();
		}

		protected override void OnSourceFilesReceived(string list)
		{
			Action sourceFilesReceivedAction = () =>
			{
				var relativeFileNames = new List<string>();
				relativeFileNames.AddRange(list.Split(new char[] { '|' }, StringSplitOptions.RemoveEmptyEntries));

				var fileNames = new List<string>();
				relativeFileNames.Sort(new Comparison<string>((first, second) =>
				{
					first = Path.GetFileName(first);
					second = Path.GetFileName(second);

					int result = Comparer<string>.Default.Compare(first, second);
					if (result != 0)
					{
						if (first.Equals("Example.xaml", StringComparison.OrdinalIgnoreCase))
						{
							return -1;
						}
						if (second.Equals("Example.xaml", StringComparison.OrdinalIgnoreCase))
						{
							return 1;
						}
					}
					return result;
				}));

                relativeFileNames.ForEach(fileName => fileNames.Add(System.IO.Path.GetFileName(fileName)));

				for (int i = 0; i < relativeFileNames.Count; i++)
				{
					string file = relativeFileNames[i];

					CodeViewerService.Current.GetSourceFileAsync(file,
						(content) => this.AddSourceCode(System.IO.Path.GetFileName(file), content));
				}
				this.TabControl.ItemsSource = fileNames;
			};

			Dispatcher.BeginInvoke(DispatcherPriority.Normal, sourceFilesReceivedAction);
		}

        protected void AddSourceCode(string file, string content)
        {
            if (content == null)
            {
                content = "Cannot find description";
            }
            if (!this.sourceCode.ContainsKey(file))
            {
                this.sourceCode.Add(file, ConvertTabsToSpaces(content, 4));
            }
            Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(() => DisplaySelectedFile()));
        }

		private static string GetSourceFileExtension(string fileName)
		{
			string extension = System.IO.Path.GetExtension(fileName);
			if (string.IsNullOrEmpty(extension))
			{
				return ".txt";
			}
			return extension;
		}
	}
}
