﻿using System.Collections.Generic;
using System.ComponentModel;
using Telerik.Windows.Controls;

namespace Telerik.Windows.QuickStart
{
	public class ThemesViewModel : INotifyPropertyChanged
	{
		public ThemesViewModel()
		{
			if (StyleManager.ApplicationTheme != null)
			{
				this.selectedThemeIndex = Themes.IndexOf(StyleManager.ApplicationTheme.ToString());
			}
			else
			{
#if WPF
				this.selectedThemeIndex = 0;
#else
				this.selectedThemeIndex = -1;
#endif
			}
		}

		private int selectedThemeIndex;

		public int SelectedThemeIndex
		{
			get
			{
				return this.selectedThemeIndex;
			}
			set
			{
				if (this.selectedThemeIndex != value)
				{
					this.selectedThemeIndex = value;
					StyleManager.ApplicationTheme = ThemeManager.FromName(Themes[value]);
					this.OnPropertyChanged("SelectedThemeIndex");
				}
			}
		}

		public static IList<string> Themes
		{
			get
			{
#if WPF
				if (!ThemeManager.StandardThemeNames.Contains("Themes"))
				{
					ThemeManager.StandardThemeNames.Insert(0, "Themes");
				}
#endif
				return ThemeManager.StandardThemeNames;
			}
		}

		public event PropertyChangedEventHandler PropertyChanged;

		private void OnPropertyChanged(string propertyName)
		{
			if (this.PropertyChanged != null)
			{
				this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}
	}
}