﻿using System;
using System.Resources;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Resources;
using System.Xml.Linq;
using Telerik.Windows.Controls;
using System.Collections.Generic;
using System.Windows.Browser;

namespace Telerik.Windows.QuickStart
{
	public partial class Default : UserControl
	{
		private Type examplesApplicationType;

		public Default()
		{
			InitializeComponent();

			Navigation = NavigationService.GetNavigationService(this.ExampleContainer);

			ExamplesTree.SelectionChanged += new Telerik.Windows.Controls.SelectionChangedEventHandler(ExamplesTree_SelectionChanged);
            
			Loaded += new RoutedEventHandler(Default_Loaded);
		}

		void Default_Loaded(object sender, RoutedEventArgs e)
		{
			string bookmark = HtmlPage.Window.CurrentBookmark;
			if (String.IsNullOrEmpty(bookmark))
			{
				bookmark = DefaultExample;
			}
			Navigation.Navigate(GetShortExampleTypeName(bookmark));
		}

		private string ExampleApplicationRootNamespace
		{
			get
			{
				return examplesApplicationType.Namespace;
			}
		}

		private string ExampleApplicationAssemblyName
		{
			get
			{
				return examplesApplicationType.AssemblyQualifiedName.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries)[1].Trim();
			}
		}

		public void LoadExamples(Type type)
		{
			examplesApplicationType = type;

			StreamResourceInfo resource = Application.GetResourceStream(new Uri(string.Format("/{0};component/Examples.xml", ExampleApplicationAssemblyName), UriKind.Relative));
			if (resource == null)
			{
				throw new MissingManifestResourceException("Cannot find Examples.xml");
			}
			XDocument document = XDocument.Load(resource.Stream);

			IEnumerable<ExampleItem> examples = ExampleItem.SelectAll(document.Element("examples"));

			RegisterExamplePages(examples);

			ExamplesTree.ItemsSource = examples;
		}

		private NavigationService Navigation;

		private Type GetExampleType(string itemPath)
		{
			return Type.GetType(examplesApplicationType.AssemblyQualifiedName.Replace(examplesApplicationType.FullName, GetShortExampleTypeName(itemPath)));
		}

		private string GetShortExampleTypeName(string itemPath)
		{
			return ExampleApplicationRootNamespace + ".Examples." + itemPath.Replace('/', '.') + ".Example";
		}

		private string DefaultExample { get; set; }

        private void RegisterExamplePages(IEnumerable<ExampleItem> items)
        {
            foreach (ExampleItem item in items)
            {
                if (item.IsExample)
                {
                    Type exampleType = GetExampleType(item.Path);
                    if (exampleType != null)
                    {
                        Example example = Activator.CreateInstance(exampleType) as Example;
                        example.ExampleItem = item;

                        if (DefaultExample == null)
                        {
                            DefaultExample = item.Path;
                        }

                        Navigation.RegisterFrame(example);
                    }
                }
                else
                {
                    RegisterExamplePages(item.Children);
                }
            }
        }

		private void ExamplesTree_SelectionChanged(object sender, Telerik.Windows.Controls.SelectionChangedEventArgs e)
		{
			ExampleItem item = ExamplesTree.SelectedItem as ExampleItem;
			if (item.IsExample)
			{
				Navigation.Navigate(Navigation.GetPage(GetShortExampleTypeName(item.Path)));

				HtmlPage.Window.CurrentBookmark = item.Path;
			}
		}
	}
}