﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Data;

namespace Telerik.Windows.QuickStart
{
	public class ExamplesCollection : ObservableCollection<ExampleItem>
	{
		public ExamplesCollection()
		{
		}

		internal ExamplesCollection(List<ExampleItem> listItems)
			: this()
		{
            listItems.ForEach(exampleItem => this.Add(exampleItem));
		}

		public void AddRange(ExamplesCollection exampleCollection)
		{
			foreach (ExampleItem item in exampleCollection)
			{
				this.Add(item);
			}
		}

		internal void ApplyFilter(string value)
		{
			foreach (ExampleItem item in this)
			{
				item.ApplyFilter(value);
			}
		}

		private void GetFirstValidResult(ref ExampleItem searchItem)
		{
			foreach (ExampleItem item in this)
			{
				if (item.IsExample && item.Visible)
				{
					if (searchItem == null)
					{
						searchItem = item;
					}
				}
				else
				{
					item.Children.GetFirstValidResult(ref searchItem);
				}
			}
		}

		internal ExampleItem SelectFirstValidOrDefault(ExampleItem defaultItem)
		{
			ExampleItem resultItem = null;
			GetFirstValidResult(ref resultItem);

			return resultItem ?? defaultItem;
		}

		internal void ExpandedAll(bool value)
		{
			foreach (ExampleItem item in this)
			{
				item.IsExpanded = value;
			}
		}
	}

	public class BooleanToVisibilityConverter : IValueConverter
	{
		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			return (bool)value ? Visibility.Visible : Visibility.Collapsed;
		}

		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			throw new NotImplementedException();
		}
	}
}
