﻿using System.Windows;
using System;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Controls.Primitives;
using System.Windows.Automation.Peers;
using System.Collections;

namespace Telerik.Windows.QuickStart
{
    [TemplateVisualState(Name = "Disabled", GroupName = "CommonStates"),
            TemplatePart(Name = "HorizontalTemplate", Type = typeof(FrameworkElement)),
            TemplatePart(Name = "VerticalTemplate", Type = typeof(FrameworkElement)),
     TemplateVisualState(Name = "Normal", GroupName = "CommonStates"),
     TemplateVisualState(Name = "MouseOver", GroupName = "CommonStates"),
     TemplateVisualState(Name = "Focused", GroupName = "FocusStates"),
     TemplateVisualState(Name = "Unfocused", GroupName = "FocusStates")]

    public class GridSplitter : Control
    {
        // Fields
        private GridResizeDirection _currentGridResizeDirection;
        private DragValidator _dragValidator;
        internal FrameworkElement _elementHorizontalTemplateFrameworkElement;
        internal FrameworkElement _elementVerticalTemplateFrameworkElement;
        private bool _isMouseOver;
        private Canvas _previewLayer;
        internal ResizeData _resizeData;
        private InternalTestHook _testHook;
        private const double DragIncrement = 1.0;
        internal const string ElementHorizontalTemplateName = "HorizontalTemplate";
        internal const string ElementVerticalTemplateName = "VerticalTemplate";
        private const double KeyboardIncrement = 10.0;
        public static readonly DependencyProperty PreviewStyleProperty = DependencyProperty.Register("PreviewStyle", typeof(Style), typeof(GridSplitter), null);
        public static readonly DependencyProperty ShowsPreviewProperty = DependencyProperty.Register("ShowsPreview", typeof(bool), typeof(GridSplitter), null);

        // Methods
        public GridSplitter()
        {
            EventHandler handler = null;
            MouseEventHandler handler2 = null;
            MouseEventHandler handler3 = null;
            RoutedEventHandler handler4 = null;
            RoutedEventHandler handler5 = null;

            base.IsEnabledChanged += new DependencyPropertyChangedEventHandler(this.OnIsEnabledChanged);
            base.KeyDown += new KeyEventHandler(this.GridSplitter_KeyDown);
            if (handler == null)
            {
                handler = delegate
                {
                    this.UpdateTemplateOrientation();
                };
            }
            base.LayoutUpdated += handler;
            this._dragValidator = new DragValidator(this);
            this._dragValidator.DragStartedEvent += new EventHandler<DragStartedEventArgs>(this.DragValidator_DragStartedEvent);
            this._dragValidator.DragDeltaEvent += new EventHandler<DragDeltaEventArgs>(this.DragValidator_DragDeltaEvent);
            this._dragValidator.DragCompletedEvent += new EventHandler<DragCompletedEventArgs>(this.DragValidator_DragCompletedEvent);
            if (handler2 == null)
            {
                handler2 = delegate(object sender, MouseEventArgs e)
                {
                    this._isMouseOver = true;
                    this.ChangeVisualState();
                };
            }
            base.MouseEnter += handler2;
            if (handler3 == null)
            {
                handler3 = delegate(object sender, MouseEventArgs e)
                {
                    this._isMouseOver = false;
                    if (this._resizeData == null)
                    {
                        this.ChangeVisualState();
                    }
                };
            }
            base.MouseLeave += handler3;
            if (handler4 == null)
            {
                handler4 = delegate(object sender, RoutedEventArgs e)
                {
                    this.ChangeVisualState();
                };
            }
            base.GotFocus += handler4;
            if (handler5 == null)
            {
                handler5 = delegate(object sender, RoutedEventArgs e)
                {
                    this.ChangeVisualState();
                };
            }
            base.LostFocus += handler5;
            base.DefaultStyleKey = typeof(GridSplitter);
        }

        private void CancelResize()
        {
            if (this._resizeData.ShowsPreview)
            {
                this.RemovePreviewControl();
            }
            else
            {
                this.SetLengths(this._resizeData.OriginalDefinition1ActualLength, this._resizeData.OriginalDefinition2ActualLength);
            }
            this._resizeData = null;
        }

        private void ChangeVisualState()
        {
            this.ChangeVisualState(true);
        }

        private void ChangeVisualState(bool useTransitions)
        {
            if (!base.IsEnabled)
            {
                VisualStates.GoToState(this, useTransitions, new string[] { "Disabled", "Normal" });
            }
            else if (this._isMouseOver)
            {
                VisualStates.GoToState(this, useTransitions, new string[] { "MouseOver", "Normal" });
            }
            else
            {
                VisualStates.GoToState(this, useTransitions, new string[] { "Normal" });
            }
            if (this.HasKeyboardFocus && base.IsEnabled)
            {
                VisualStates.GoToState(this, useTransitions, new string[] { "Focused", "Unfocused" });
            }
            else
            {
                VisualStates.GoToState(this, useTransitions, new string[] { "Unfocused" });
            }
            if (this.GetEffectiveResizeDirection() == GridResizeDirection.Columns)
            {
                base.Cursor = Cursors.SizeWE;
            }
            else
            {
                base.Cursor = Cursors.SizeNS;
            }
        }

        private void CreatePreviewLayer(Grid parentGrid)
        {
            this._previewLayer = new Canvas();
            if (parentGrid.RowDefinitions.Count > 0)
            {
                this._previewLayer.SetValue(Grid.RowSpanProperty, parentGrid.RowDefinitions.Count);
            }
            if (parentGrid.ColumnDefinitions.Count > 0)
            {
                this._previewLayer.SetValue(Grid.ColumnSpanProperty, parentGrid.ColumnDefinitions.Count);
            }
            parentGrid.Children.Add(this._previewLayer);
        }

        internal void DragValidator_DragCompletedEvent(object sender, DragCompletedEventArgs e)
        {
            if (this._resizeData != null)
            {
                if (e.Canceled)
                {
                    this.CancelResize();
                }
                else if (this._resizeData.ShowsPreview)
                {
                    this.MoveSplitter(this._resizeData.PreviewControl.OffsetX, this._resizeData.PreviewControl.OffsetY);
                    this.RemovePreviewControl();
                }
                this._resizeData = null;
            }
            this.ChangeVisualState();
        }

        internal void DragValidator_DragDeltaEvent(object sender, DragDeltaEventArgs e)
        {
            if (this._resizeData != null)
            {
                double horizontalChange = e.HorizontalChange;
                double verticalChange = e.VerticalChange;
                if (this._resizeData.ShowsPreview)
                {
                    if (this._resizeData.ResizeDirection == GridResizeDirection.Columns)
                    {
                        this._resizeData.PreviewControl.OffsetX = Math.Min(Math.Max(horizontalChange, this._resizeData.MinChange), this._resizeData.MaxChange);
                    }
                    else
                    {
                        this._resizeData.PreviewControl.OffsetY = Math.Min(Math.Max(verticalChange, this._resizeData.MinChange), this._resizeData.MaxChange);
                    }
                }
                else
                {
                    this.MoveSplitter(horizontalChange, verticalChange);
                }
            }
        }

        internal void DragValidator_DragStartedEvent(object sender, DragStartedEventArgs e)
        {
            if (base.IsEnabled)
            {
                base.Focus();
                this.InitializeData(this.ShowsPreview);
            }
        }

        private static double GetActualLength(DefinitionAbstraction definition)
        {
            if (definition.AsColumnDefinition != null)
            {
                return definition.AsColumnDefinition.ActualWidth;
            }
            return definition.AsRowDefinition.ActualHeight;
        }

        private double[] GetDeltaConstraints()
        {
            double num7;
            double num8;
            double actualLength = GetActualLength(this._resizeData.Definition1);
            double minSize = this._resizeData.Definition1.MinSize;
            double maxSize = this._resizeData.Definition1.MaxSize;
            double num4 = GetActualLength(this._resizeData.Definition2);
            double num5 = this._resizeData.Definition2.MinSize;
            double num6 = this._resizeData.Definition2.MaxSize;

            if (this._resizeData.SplitterIndex == this._resizeData.Definition1Index)
            {
                minSize = Math.Max(minSize, this._resizeData.SplitterLength);
            }
            else if (this._resizeData.SplitterIndex == this._resizeData.Definition2Index)
            {
                num5 = Math.Max(num5, this._resizeData.SplitterLength);
            }
            if (this._resizeData.SplitBehavior == SplitBehavior.Split)
            {
                num7 = -Math.Min((double)(actualLength - minSize), (double)(num6 - num4));
                num8 = Math.Min((double)(maxSize - actualLength), (double)(num4 - num5));
            }
            else if (this._resizeData.SplitBehavior == SplitBehavior.ResizeDefinition1)
            {
                num7 = minSize - actualLength;
                num8 = maxSize - actualLength;
            }
            else
            {
                num7 = num4 - num6;
                num8 = num4 - num5;
            }
            return new double[] { num7, num8 };
        }

        private GridResizeBehavior GetEffectiveResizeBehavior(GridResizeDirection direction)
        {
            if (direction == GridResizeDirection.Columns)
            {
                switch (base.HorizontalAlignment)
                {
                    case HorizontalAlignment.Left:
                        return GridResizeBehavior.PreviousAndCurrent;

                    case HorizontalAlignment.Center:
                        goto Label_0044;

                    case HorizontalAlignment.Right:
                        return GridResizeBehavior.CurrentAndNext;
                }
            }
            else
            {
                switch (base.VerticalAlignment)
                {
                    case VerticalAlignment.Top:
                        return GridResizeBehavior.PreviousAndCurrent;

                    case VerticalAlignment.Bottom:
                        return GridResizeBehavior.CurrentAndNext;
                }
                return GridResizeBehavior.PreviousAndNext;
            }
        Label_0044:
            return GridResizeBehavior.PreviousAndNext;
        }

        private GridResizeDirection GetEffectiveResizeDirection()
        {
            if (base.HorizontalAlignment != HorizontalAlignment.Stretch)
            {
                return GridResizeDirection.Columns;
            }
            if ((base.VerticalAlignment == VerticalAlignment.Stretch) && (base.ActualWidth <= base.ActualHeight))
            {
                return GridResizeDirection.Columns;
            }
            return GridResizeDirection.Rows;
        }

        private static DefinitionAbstraction GetGridDefinition(Grid grid, int index, GridResizeDirection direction)
        {
            if (direction != GridResizeDirection.Columns)
            {
                return new DefinitionAbstraction(grid.RowDefinitions[index]);
            }
            return new DefinitionAbstraction(grid.ColumnDefinitions[index]);
        }

        internal void GridSplitter_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.Left:
                    e.Handled = this.KeyboardMoveSplitter(-10.0, 0.0);
                    return;

                case Key.Up:
                    e.Handled = this.KeyboardMoveSplitter(0.0, -10.0);
                    return;

                case Key.Right:
                    e.Handled = this.KeyboardMoveSplitter(10.0, 0.0);
                    return;

                case Key.Down:
                    e.Handled = this.KeyboardMoveSplitter(0.0, 10.0);
                    return;

                case Key.Escape:
                    if (this._resizeData != null)
                    {
                        this.CancelResize();
                        e.Handled = true;
                    }
                    return;
            }
        }

        internal bool InitializeAndMoveSplitter(double horizontalChange, double verticalChange)
        {
            if (this._resizeData != null)
            {
                return false;
            }
            this.InitializeData(false);
            if (this._resizeData == null)
            {
                return false;
            }
            this.MoveSplitter(horizontalChange, verticalChange);
            this._resizeData = null;
            return true;
        }

        private void InitializeData(bool showsPreview)
        {
            Grid parent = base.Parent as Grid;
            if (parent != null)
            {
                this._resizeData = new ResizeData();
                this._resizeData.Grid = parent;
                this._resizeData.ShowsPreview = showsPreview;
                this._resizeData.ResizeDirection = this.GetEffectiveResizeDirection();
                this._resizeData.ResizeBehavior = this.GetEffectiveResizeBehavior(this._resizeData.ResizeDirection);
                this._resizeData.SplitterLength = Math.Min(base.ActualWidth, base.ActualHeight);
                if (!this.SetupDefinitionsToResize())
                {
                    this._resizeData = null;
                }
                else
                {
                    this.SetupPreview();
                }
            }
        }

        private static bool IsStar(DefinitionAbstraction definition)
        {
            if (definition.AsColumnDefinition != null)
            {
                return definition.AsColumnDefinition.Width.IsStar;
            }
            return definition.AsRowDefinition.Height.IsStar;
        }

        private bool KeyboardMoveSplitter(double horizontalChange, double verticalChange)
        {
            return ((this.HasKeyboardFocus && base.IsEnabled) && this.InitializeAndMoveSplitter(horizontalChange, verticalChange));
        }

        private void MoveSplitter(double horizontalChange, double verticalChange)
        {
            double num = (this._resizeData.ResizeDirection == GridResizeDirection.Columns) ? horizontalChange : verticalChange;
            DefinitionAbstraction definition = this._resizeData.Definition1;
            DefinitionAbstraction abstraction2 = this._resizeData.Definition2;
            if ((definition != null) && (abstraction2 != null))
            {
                double actualLength = GetActualLength(definition);
                double num3 = GetActualLength(abstraction2);
                if ((this._resizeData.SplitBehavior == SplitBehavior.Split) && !DoubleUtil.AreClose(actualLength + num3, this._resizeData.OriginalDefinition1ActualLength + this._resizeData.OriginalDefinition2ActualLength))
                {
                    this.CancelResize();
                }
                else
                {
                    double[] deltaConstraints = this.GetDeltaConstraints();
                    double num4 = deltaConstraints[0];
                    double num5 = deltaConstraints[1];
                    num = Math.Min(Math.Max(num, num4), num5);
                    double num6 = actualLength + num;
                    double num7 = num3 - num;
                    this.SetLengths(num6, num7);
                }
            }
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            this._elementHorizontalTemplateFrameworkElement = base.GetTemplateChild("HorizontalTemplate") as FrameworkElement;
            this._elementVerticalTemplateFrameworkElement = base.GetTemplateChild("VerticalTemplate") as FrameworkElement;
            this._currentGridResizeDirection = GridResizeDirection.Auto;
            this.UpdateTemplateOrientation();
            this.ChangeVisualState(false);
        }

        private void OnIsEnabledChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            if (!((bool)e.NewValue))
            {
                this._isMouseOver = false;
            }
            this.ChangeVisualState();
        }

        private void RemovePreviewControl()
        {
            if ((this._resizeData.PreviewControl != null) && (this._previewLayer != null))
            {
                this._previewLayer.Children.Remove(this._resizeData.PreviewControl);
            }
        }

        private static void SetDefinitionLength(DefinitionAbstraction definition, GridLength length)
        {
            if (definition.AsColumnDefinition != null)
            {
                definition.AsColumnDefinition.SetValue(ColumnDefinition.WidthProperty, length);
            }
            else
            {
                definition.AsRowDefinition.SetValue(RowDefinition.HeightProperty, length);
            }
        }

        private void SetLengths(double definition1Pixels, double definition2Pixels)
        {
            if (this._resizeData.SplitBehavior == SplitBehavior.Split)
            {
                IEnumerable enumerable = (this._resizeData.ResizeDirection == GridResizeDirection.Columns) ? ((IEnumerable)this._resizeData.Grid.ColumnDefinitions) : ((IEnumerable)this._resizeData.Grid.RowDefinitions);
                int num = 0;
                foreach (DependencyObject obj2 in enumerable)
                {
                    DefinitionAbstraction definition = new DefinitionAbstraction(obj2);
                    if (num == this._resizeData.Definition1Index)
                    {
                        SetDefinitionLength(definition, new GridLength(definition1Pixels, GridUnitType.Star));
                    }
                    else if (num == this._resizeData.Definition2Index)
                    {
                        SetDefinitionLength(definition, new GridLength(definition2Pixels, GridUnitType.Star));
                    }
                    else if (IsStar(definition))
                    {
                        SetDefinitionLength(definition, new GridLength(GetActualLength(definition), GridUnitType.Star));
                    }
                    num++;
                }
            }
            else if (this._resizeData.SplitBehavior == SplitBehavior.ResizeDefinition1)
            {
                SetDefinitionLength(this._resizeData.Definition1, new GridLength(definition1Pixels));
            }
            else
            {
                SetDefinitionLength(this._resizeData.Definition2, new GridLength(definition2Pixels));
            }
        }

        private bool SetupDefinitionsToResize()
        {
            int num = (int)base.GetValue((this._resizeData.ResizeDirection == GridResizeDirection.Columns) ? Grid.ColumnSpanProperty : Grid.RowSpanProperty);
            if (num == 1)
            {
                int num2;
                int num3;
                int num4 = (int)base.GetValue((this._resizeData.ResizeDirection == GridResizeDirection.Columns) ? Grid.ColumnProperty : Grid.RowProperty);
                switch (this._resizeData.ResizeBehavior)
                {
                    case GridResizeBehavior.CurrentAndNext:
                        num2 = num4;
                        num3 = num4 + 1;
                        break;

                    case GridResizeBehavior.PreviousAndCurrent:
                        num2 = num4 - 1;
                        num3 = num4;
                        break;

                    default:
                        num2 = num4 - 1;
                        num3 = num4 + 1;
                        break;
                }
                int num5 = (this._resizeData.ResizeDirection == GridResizeDirection.Columns) ? this._resizeData.Grid.ColumnDefinitions.Count : this._resizeData.Grid.RowDefinitions.Count;
                if ((num2 >= 0) && (num3 < num5))
                {
                    this._resizeData.SplitterIndex = num4;
                    this._resizeData.Definition1Index = num2;
                    this._resizeData.Definition1 = GetGridDefinition(this._resizeData.Grid, num2, this._resizeData.ResizeDirection);
                    this._resizeData.OriginalDefinition1Length = this._resizeData.Definition1.Size;
                    this._resizeData.OriginalDefinition1ActualLength = GetActualLength(this._resizeData.Definition1);
                    this._resizeData.Definition2Index = num3;
                    this._resizeData.Definition2 = GetGridDefinition(this._resizeData.Grid, num3, this._resizeData.ResizeDirection);
                    this._resizeData.OriginalDefinition2Length = this._resizeData.Definition2.Size;
                    this._resizeData.OriginalDefinition2ActualLength = GetActualLength(this._resizeData.Definition2);
                    bool flag = IsStar(this._resizeData.Definition1);
                    bool flag2 = IsStar(this._resizeData.Definition2);
                    if (flag && flag2)
                    {
                        this._resizeData.SplitBehavior = SplitBehavior.Split;
                    }
                    else
                    {
                        this._resizeData.SplitBehavior = !flag ? SplitBehavior.ResizeDefinition1 : SplitBehavior.ResizeDefinition2;
                    }
                    return true;
                }
            }
            return false;
        }

        private void SetupPreview()
        {
            if (this._resizeData.ShowsPreview)
            {
                if (this._previewLayer == null)
                {
                    this.CreatePreviewLayer(this._resizeData.Grid);
                }
                this._resizeData.PreviewControl = new PreviewControl();
                this._resizeData.PreviewControl.Bind(this);
                this._previewLayer.Children.Add(this._resizeData.PreviewControl);
                double[] deltaConstraints = this.GetDeltaConstraints();
                this._resizeData.MinChange = deltaConstraints[0];
                this._resizeData.MaxChange = deltaConstraints[1];
            }
        }

        private void UpdateTemplateOrientation()
        {
            GridResizeDirection effectiveResizeDirection = this.GetEffectiveResizeDirection();
            if (this._currentGridResizeDirection != effectiveResizeDirection)
            {
                if (effectiveResizeDirection == GridResizeDirection.Columns)
                {
                    if (this._elementHorizontalTemplateFrameworkElement != null)
                    {
                        this._elementHorizontalTemplateFrameworkElement.Visibility = Visibility.Collapsed;
                    }
                    if (this._elementVerticalTemplateFrameworkElement != null)
                    {
                        this._elementVerticalTemplateFrameworkElement.Visibility = Visibility.Visible;
                    }
                }
                else
                {
                    if (this._elementHorizontalTemplateFrameworkElement != null)
                    {
                        this._elementHorizontalTemplateFrameworkElement.Visibility = Visibility.Visible;
                    }
                    if (this._elementVerticalTemplateFrameworkElement != null)
                    {
                        this._elementVerticalTemplateFrameworkElement.Visibility = Visibility.Collapsed;
                    }
                }
                this._currentGridResizeDirection = effectiveResizeDirection;
            }
        }

        // Properties
        private bool HasKeyboardFocus
        {
            get
            {
                return (FocusManager.GetFocusedElement() == this);
            }
        }

        public Style PreviewStyle
        {
            get
            {
                return (Style)base.GetValue(PreviewStyleProperty);
            }
            set
            {
                base.SetValue(PreviewStyleProperty, value);
            }
        }

        public bool ShowsPreview
        {
            get
            {
                return (bool)base.GetValue(ShowsPreviewProperty);
            }
            set
            {
                base.SetValue(ShowsPreviewProperty, value);
            }
        }

        internal InternalTestHook TestHook
        {
            get
            {
                if (this._testHook == null)
                {
                    this._testHook = new InternalTestHook(this);
                }
                return this._testHook;
            }
        }

        // Nested Types
        internal class DefinitionAbstraction
        {

            // Methods
            public DefinitionAbstraction(DependencyObject definition)
            {
                Grid g = new Grid();
                g.ColumnDefinitions.Add(new System.Windows.Controls.ColumnDefinition());
                this.AsRowDefinition = definition as RowDefinition;
                if (this.AsRowDefinition == null)
                {
                    this.AsColumnDefinition = definition as ColumnDefinition;
                }
            }

            // Properties
            public ColumnDefinition AsColumnDefinition
            {
                get;
                set;
            }

            public RowDefinition AsRowDefinition
            {
                get;
                set;
            }

            public double MaxSize
            {
                get
                {
                    if (this.AsRowDefinition != null)
                    {
                        return this.AsRowDefinition.MaxHeight;
                    }
                    return this.AsColumnDefinition.MaxWidth;
                }
            }

            public double MinSize
            {
                get
                {
                    if (this.AsRowDefinition != null)
                    {
                        return this.AsRowDefinition.MinHeight;
                    }
                    return this.AsColumnDefinition.MinWidth;
                }
            }

            public GridLength Size
            {
                get
                {
                    if (this.AsRowDefinition != null)
                    {
                        return this.AsRowDefinition.Height;
                    }
                    return this.AsColumnDefinition.Width;
                }
            }
        }

        internal static class DoubleUtil
        {
            // Fields
            private const double Epsilon = 1.192093E-07;
            private const double ScalarAdjustment = 10.0;

            // Methods
            public static bool AreClose(double value1, double value2)
            {
                if (value1 == value2)
                {
                    return true;
                }
                double num = ((Math.Abs(value1) + Math.Abs(value2)) + 10.0) * 1.192093E-07;
                double num2 = value1 - value2;
                return ((-num < num2) && (num > num2));
            }
        }

        internal enum GridResizeBehavior
        {
            BasedOnAlignment,
            CurrentAndNext,
            PreviousAndCurrent,
            PreviousAndNext
        }

        internal enum GridResizeDirection
        {
            Auto,
            Columns,
            Rows
        }

        internal class InternalTestHook
        {
            // Fields
            private GridSplitter _gridSplitter;

            // Methods
            internal InternalTestHook(GridSplitter gridSplitter)
            {
                this._gridSplitter = gridSplitter;
            }

            internal void DragValidator_DragCompletedEvent(object sender, DragCompletedEventArgs e)
            {
                this._gridSplitter.DragValidator_DragCompletedEvent(sender, e);
            }

            internal void DragValidator_DragDeltaEvent(object sender, DragDeltaEventArgs e)
            {
                this._gridSplitter.DragValidator_DragDeltaEvent(sender, e);
            }

            internal void DragValidator_DragStartedEvent(object sender, DragStartedEventArgs e)
            {
                this._gridSplitter.DragValidator_DragStartedEvent(sender, e);
            }

            internal bool KeyboardMoveSplitter(double horizontalChange, double verticalChange)
            {
                return this._gridSplitter.KeyboardMoveSplitter(horizontalChange, verticalChange);
            }

            // Properties
            internal GridSplitter.GridResizeDirection GridResizeDirection
            {
                get
                {
                    return this._gridSplitter._currentGridResizeDirection;
                }
            }

            internal Canvas PreviewLayer
            {
                get
                {
                    return this._gridSplitter._previewLayer;
                }
            }

            internal GridSplitter.ResizeData ResizeData
            {
                get
                {
                    return this._gridSplitter._resizeData;
                }
            }
        }

        internal class ResizeData
        {
            // Fields
            public GridSplitter.DefinitionAbstraction Definition1;
            public int Definition1Index;
            public GridSplitter.DefinitionAbstraction Definition2;
            public int Definition2Index;
            public Grid Grid;
            public double MaxChange;
            public double MinChange;
            public double OriginalDefinition1ActualLength;
            public GridLength OriginalDefinition1Length;
            public double OriginalDefinition2ActualLength;
            public GridLength OriginalDefinition2Length;
            public PreviewControl PreviewControl;
            public GridSplitter.GridResizeBehavior ResizeBehavior;
            public GridSplitter.GridResizeDirection ResizeDirection;
            public bool ShowsPreview;
            public GridSplitter.SplitBehavior SplitBehavior;
            public int SplitterIndex;
            public double SplitterLength;
        }

        internal enum SplitBehavior
        {
            Split,
            ResizeDefinition1,
            ResizeDefinition2
        }
    }
}
