﻿using System.Collections.ObjectModel;
using System.Text;
using System.Windows.Media;

namespace Telerik.Windows.QuickStart.Parser
{
	internal class VBSyntaxParser : SyntaxParser
	{
		private string[] keywords = new string[]
		{
            "AddHandler",
            "AddressOf",
            "Alias",
            "AndAlso",
            "And",
            "As",
            "Boolean",
            "ByRef",
            "Byte",
            "ByVal",
            "Call",
            "Case",
            "Catch",
            "CBool",
            "CByte",
            "CChar",
            "CDate",
            "CDec",
            "CDbl",
            "Char",
            "CInt",
            "Class",
            "CLng",
            "CObj",
            "Const",
            "Continue",
            "CSByte",
            "CShort",
            "CSng",
            "CStr",
            "CType",
            "CUInt",
            "CULng",
            "CUShort",
            "Date",
            "Decimal",
            "Declare",
            "Default",
            "Delegate",
            "Dim",
            "DirectCast",
            "Double",
            "Do",
            "Each",
            "ElseIf",
            "Else",
            "EndIf",
            "End",
            "Enum",
            "Erase",
            "Error",
            "Event",
            "Exit",
            "False",
            "Finally",
            "For",
            "Friend",
            "Function",
            "GetType",
            "Get",
            "Global",
            "GoSub",
            "GoTo",
            "Handles",
            "If",
            "Implements",
            "Imports",
            "Inherits",
            "Integer",
            "Interface",
            "In",
            "IsNot",
            "Is",
            "Let",
            "Lib",
            "Like",
            "Long",
            "Loop",
            "Me",
            "Module",
            "Mod",
            "MustInherit",
            "MustOverride",
            "MyBase",
            "MyClass",
            "Namespace",
            "Narrowing",
            "New",
            "Next",
            "Nothing",
            "Not",
            "NotInheritable",
            "NotOverridable",
            "Object",
            "Of",
            "On",
            "Operator",
            "Option",
            "Optional",
            "OrElse",
            "Or",
            "Overloads",
            "Overridable",
            "Overrides",
            "ParamArray",
            "Partial",
            "Private",
            "Property",
            "Protected",
            "Public",
            "RaiseEvent",
            "ReadOnly",
            "ReDim",
            "REM",
            "RemoveHandler",
            "Resume",
            "Return",
            "SByte",
            "Select",
            "Set",
            "Shadows",
            "Shared",
            "Short",
            "Single",
            "Static",
            "Step",
            "Stop",
            "String",
            "Structure",
            "Sub",
            "SyncLock",
            "Then",
            "Throw",
            "To",
            "True",
            "Try",
            "TryCast",
            "TypeOf",
            "Variant",
            "Wend",
            "UInteger",
            "ULong",
            "UShort",
            "Using",
            "When",
            "While",
            "Widening",
            "With",
            "WithEvents",
            "WriteOnly",
            "Xor",
          ////  "#Const",
           //// "#Else",
		  // // "#ElseIf",
		  // // "#End",
		  ////  "#If",
		  // // "-",
		  // // "&",
		  // // "&=",
		  // // "*",
		  // // "*=",
		  // // "/",
		  // // "/=",
		  // // "\\",
		  // // "\\=",
		  // // "^",
		  // // "^=",
		  // // "+",
		  ////  "+=",
		  ////  "=",
		  // // "-=",
        };

		protected override Collection<LanguageSyntaxStructure> LoadLanguageSyntax(Collection<LanguageSyntaxStructure> languageSyntax)
		{
			////Load comments
			string comments = @"\G(?<comment>'[ \t\S]*\s*\s)";
			languageSyntax.Add(new LanguageSyntaxStructure(comments, "comment", new SolidColorBrush((Color)ColorConverter.ConvertFromString("#537D01"))));

			////Load Keywords
			StringBuilder kwrds = new StringBuilder();
			kwrds.Append(@"\G(?<keyword>(");
			for (int i = 0; i < this.keywords.Length; i++)
			{
				kwrds.Append(this.keywords[i] + "|");
			}
			kwrds.Append(this.keywords[this.keywords.Length - 1]);
			kwrds.Append(@")(?=(\.)|(\s+)))");
			languageSyntax.Add(new LanguageSyntaxStructure(kwrds.ToString(), "keyword", new SolidColorBrush((Color)ColorConverter.ConvertFromString("#0160E5"))));

			////Load string
			string strings = "\\G(?<string>\"\\s*((\\\\\")|[^\\\"])*\\s*\"\\s*)";
			languageSyntax.Add(new LanguageSyntaxStructure(strings, "string", new SolidColorBrush((Color)ColorConverter.ConvertFromString("#0160E5"))));

			////Load identifier
			string identifier = @"\G(?<identifier>[\w]+\s*)";
			languageSyntax.Add(new LanguageSyntaxStructure(identifier, "identifier", new SolidColorBrush((Color)ColorConverter.ConvertFromString("#000000"))));

			////Load other
			string other = @"\G(?<other>[\s\S]\s*)";
			languageSyntax.Add(new LanguageSyntaxStructure(other, "other", new SolidColorBrush((Color)ColorConverter.ConvertFromString("#000000"))));
			return base.LoadLanguageSyntax(languageSyntax);
		}

		protected override void SetFileExtension(string extension)
		{
			base.SetFileExtension(".vb");
		}
	}
}
