﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;

namespace Telerik.Windows.QuickStart.Animation
{
	public class ExampleTransition : ITransition
	{
		private Storyboard animation = new Storyboard();
		private FrameworkElement item1;
		private FrameworkElement item2;
		private FrameworkElement mask1;
		private FrameworkElement mask2;
		private bool isRunning;

		public event EventHandler<TransitionEventArgs> TransitionAnimationCompleted = delegate { };

#if WPF
		public ExampleTransition(FrameworkElement mask1, FrameworkElement mask2)
		{
#else
		public ExampleTransition(double pageWidth, Grid panel, FrameworkElement mask1, FrameworkElement mask2)
		{
			this.PageWidth = pageWidth;
			this.Container = panel;
#endif
			this.mask1 = mask1;
			this.mask2 = mask2;
		}

		public double PageWidth
		{
			get;
			set;
		}

		public Grid Container
		{
			get;
			set;
		}

		public bool IsRunning
		{
			get
			{
				return isRunning;
			}
#if !WPF
			set
			{
				throw new NotImplementedException();
			}
#endif
		}

#if WPF
		public void Begin(FrameworkElement mockItem1, Grid container, FrameworkElement newPage)
		{
			this.Container = container;
			this.PageWidth = container.ActualWidth;
			this.item1 = mockItem1;
			this.item2 = newPage;

			this.Container.Children.Add(this.item1);

			var width = PageWidth;

			var scaleFactor = 0.8;
			var halfSrink = (1.0 - scaleFactor) / 2;
#else
		public void Begin(IFrame page)
		{
			double width = this.PageWidth;
			this.item2 = page as FrameworkElement;

			if (this.Container.Children.Count > 0 && this.Container.Children[0] is RadFrame)
			{
				this.Container.Children.RemoveAt(0);
			}

			if (this.Container.Children.Count < 1)
			{
				this.Container.Children.Add(this.item2);
				return;
			}


			this.item1 = this.Container.Children[0] as FrameworkElement;

			this.Container.Children.Add(this.item2);

			var scaleFactor = 0.8;
			var page1 = item1 as IFrame;
			
			var halfSrink = (1.0 - scaleFactor) / 2;
#endif
			this.Container.Children.Add(this.mask1);
			this.Container.Children.Add(this.mask2);

			var scaleDownSpline = new Spline(0.504000008106232, 0.256000012159348, 0.458999991416931, 1);
			var moveSpline = new Spline(0.247999995946884, 0, 1, 1);
			var scaleUpSpline = new Spline(0.321000009775162, 0, 0.45100000500679, 1);
			var spacing = 15.0;

			this.animation = this.item1
				.Animate()
				.With(this.mask1)
					.EnsureDefaultTransforms()
					.Origin(0, 0.5)
					.Scale(0, 1, 0.2, 1, 0.7, scaleFactor)
						.Splines(2, scaleDownSpline)
					.MoveX(0.7, 0, 1.365, (-width * (scaleFactor - halfSrink)) - spacing, 1.9, (-width * scaleFactor) - spacing)
						.Splines(1, moveSpline)
						.Splines(2, scaleUpSpline)
				.Without(this.item1)
					.Opacity(0, 0, 0.2, 0, 0.7, 1)
						.Splines(1, scaleDownSpline)
				.Animate(this.mask2)
					.Opacity(1.35, 1, 1.9, 0)
						.Splines(scaleUpSpline)
				.With(this.item2)
					.EnsureDefaultTransforms()
					.Origin(0.5, 0.5)
					.Scale(0, scaleFactor, 1.365, scaleFactor, 1.9, 1)
						.Splines(1, moveSpline)
						.Splines(2, scaleUpSpline)
					.MoveX(0.0, ((scaleFactor + halfSrink) * width) + spacing, 0.2, ((scaleFactor + halfSrink) * width) + spacing, 0.7, ((scaleFactor - halfSrink) * width) + spacing, 1.365, 0)
						.Splines(2, scaleDownSpline)
						.Splines(3, moveSpline)
				.Instance;

			this.animation.SpeedRatio = 755.0 / width * 1.5;

			this.isRunning = true;
#if WPF
			this.animation.Completed += (sender, e) =>
			{
                this.TransitionAnimationCompleted(this, new TransitionEventArgs(item2));

				Container.Children.Remove(mask1);
				Container.Children.Remove(mask2);
				RemoveVisualBrush();
				Container.Children.Remove(item1);
				item1 = null;
				item2 = null;
			};
#else
			this.animation.Completed += (sender, e) =>
			{
				this.TransitionAnimationCompleted(this, new TransitionEventArgs(item2));

				// Unregister!
				Container.Children.Remove(mask1);
				Container.Children.Remove(mask2);
				this.Container.Children.Remove(this.item1);
				item1 = null;
				item2 = null;

			};
#endif
			this.animation.Begin();
		}

#if WPF
		private void RemoveVisualBrush()
		{
			var shape = item1 as Shape;
			if (shape != null)
			{
				var visBrush = shape.Fill as VisualBrush;
				if (visBrush != null)
				{
					visBrush.Visual = null;
				}
			}
		}
#endif

		public void StopTransition()
		{
			// Move to initial state:
			this.animation.Stop();
#if WPF
			this.Container.Children.Remove(this.item1);
#else
			this.Container.Children.Remove(this.item1);
#endif
			this.Container.Children.Remove(this.mask1);
			this.Container.Children.Remove(this.mask2);
			this.item1 = null;
			this.item2 = null;
			this.isRunning = false;
		}
	}

	public class TransitionEventArgs : EventArgs
	{
		public FrameworkElement TargetExamplePage
		{
			get;
			set;
		}

		public TransitionEventArgs(FrameworkElement targetExample)
		{
			this.TargetExamplePage = targetExample;
		}
	}
}
