﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Animation;

namespace Telerik.Windows.QuickStart.Animation
{
	/// <summary>
	/// Represents the loading animation.
	/// </summary>
	/// <remarks>
	///		The animation is in separate user control, because it is quite large.
	/// </remarks>
	public partial class LoadingSpin : UserControl
	{
		private Storyboard intervalTimer;
		private List<Storyboard> animations;
		private int nextAnimation;

		public LoadingSpin()
		{
			InitializeComponent();

			this.intervalTimer = new Storyboard()
			{
				Duration = new Duration(TimeSpan.FromMilliseconds(100))
			};
			this.intervalTimer.Completed += new EventHandler(this.OnIntervalTimerCompleted);

			this.animations = new List<Storyboard>()
			{
				CycleAnimation0,
				CycleAnimation1,
				CycleAnimation2,
				CycleAnimation3,
				CycleAnimation4,
				CycleAnimation5,
				CycleAnimation6,
				CycleAnimation7,
				CycleAnimation8,
				CycleAnimation9
			};
		}

		/// <summary>
		/// Start the animation.
		/// </summary>
		public void Begin()
		{
			this.Visibility = Visibility.Visible;
			AppearAnimation.Begin();
			this.intervalTimer.Begin();
		}

		/// <summary>
		/// Stop the animation.
		/// </summary>
		public void Stop()
		{
			this.Visibility = Visibility.Collapsed;
			AppearAnimation.Stop();
			this.nextAnimation = 0;
			this.intervalTimer.Stop();
			foreach (var item in this.animations)
			{
				item.Stop();
			}
		}

		private void OnIntervalTimerCompleted(object sender, EventArgs e)
		{
			this.animations[nextAnimation].Begin();

			// Cycle:
			this.nextAnimation = this.nextAnimation > 8 ? 0 : this.nextAnimation + 1;

			this.intervalTimer.Begin();
		}
	}
}
