﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Net;
using System.Reflection;
using System.Windows;
using System.Windows.Resources;
using System.Xml.Linq;
using System.Windows.Threading;

namespace Telerik.Windows.QuickStart
{
	internal static class ApplicationHelper
	{
		private static string exampleApplicationRootNamespace;
		private static string exampleApplicationAssemblyName;

		public static Type ApplicationType
		{
			get
			{
				return Application.Current.GetType();
			}
		}

		public static string ExampleApplicationRootNamespace
		{
			get
			{
				if (exampleApplicationRootNamespace == null)
				{
					exampleApplicationRootNamespace = ApplicationType.Namespace;
				}
				return exampleApplicationRootNamespace;
			}
		}

		public static string ExampleApplicationAssemblyName
		{
			get
			{
				if (exampleApplicationAssemblyName == null)
				{
					exampleApplicationAssemblyName = ApplicationType.AssemblyQualifiedName.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries)[1].Trim();
				}
				return exampleApplicationAssemblyName;
			}
		}

		public static string ApplicationAssemblyType
		{
			get
			{
				return Path.GetExtension(ExampleApplicationAssemblyName);
			}
		}

		public static Dispatcher Dispatcher
		{
			get
			{
#if WPF
				return Application.Current.Dispatcher;
#else
				return Deployment.Current.Dispatcher;
#endif
			}
		}

		public static string CurrentLoadedAssemblyName(string path)
		{
			string value = path;
			value = value.Split('/')[0];
			value = string.Format(CultureInfo.InvariantCulture, "Examples.{0}", value);

			return value;
		}

		private static string GetPathToXap(string pathToAssembly)
		{
			return pathToAssembly + ApplicationAssemblyType;
		}

		private static void DownloadResource(string uri, OpenReadCompletedEventHandler downloadHandler)
		{
			WebClient webClient = new WebClient();
			webClient.OpenReadAsync(new Uri(uri, UriKind.Relative));
			webClient.OpenReadCompleted += downloadHandler;
		}

		public static void LoadDynamicalThemes(string themeName, Action callBack)
		{
			themeName = string.Format("Telerik.Windows.Themes.{0}", themeName);

			string resourceUri = string.Format("themes/{0}.xap", themeName);

			var handler = new OpenReadCompletedEventHandler((sender, e) =>
			{
				if (e.Error == null)
				{
					StreamResourceInfo resultStream = new StreamResourceInfo(e.Result, "application/binnary");

					StreamResourceInfo s = Application.GetResourceStream(
#if !WPF
resultStream,
#endif
 new Uri(themeName + ".dll", UriKind.RelativeOrAbsolute));

					LoadAssemblyInDomain(s.Stream);
					callBack();
				}
			});

			if (!ApplicationCache.IsLoadedAssembly(themeName))
			{
				DownloadResource(resourceUri, handler);
			}
			else
			{
				callBack();
			}
		}

		internal static void LoadAssemblies(string path, string type, Action<object> callBack)
		{
			Example result;
			if (ApplicationCache.TryLoadingType(type, out result))
			{
				callBack(result);
			}
			else
			{
				BeginLoadAssembly(path, type, callBack);
			}
		}

		private static void BeginLoadAssembly(string path, string type, Action<object> callBack)
		{
			string pathToAssembly = CurrentLoadedAssemblyName(path);

			string pathToXap = string.Format("{0}.xap", GetPathToXap(pathToAssembly));

			var handler = new OpenReadCompletedEventHandler((sender, e) =>
			{
				if (!e.Cancelled && e.Error == null)
				{
					LoadAssembliesFromXAP(e.Result);

					Example result;
					if (ApplicationCache.TryLoadingType(type, out result))
					{
						callBack(result);
					}
				}
			});
			DownloadResource(pathToXap, handler);
		}

		private static void LoadAssembliesFromXAP(Stream xapStream)
		{
			Stream appManifestStram = GetSpecificResourceFromStream(xapStream, new Uri("AppManifest.xaml", UriKind.Relative));

			string appManifest = new StreamReader(appManifestStram).ReadToEnd();
			XElement document = XDocument.Parse(appManifest).Root;

			var items = document.Elements().Elements().ToList();
			items.ForEach(item =>
				{
					string assemblyName = item.Attribute("Source").Value;

					Stream resultStram = GetSpecificResourceFromStream(xapStream, new Uri(assemblyName, UriKind.Relative));
					LoadAssemblyInDomain(resultStram);
				});
		}

		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1801:ReviewUnusedParameters", MessageId = "resultStram")]
		private static void LoadAssemblyInDomain(Stream resultStram)
		{
#if !WPF
			if (Deployment.Current.Dispatcher.CheckAccess())
			{
				AssemblyPart part = new AssemblyPart();
				Assembly loadedAssembly = part.Load(resultStram);
				ApplicationCache.SaveAssemblyInCache(loadedAssembly);
			}
			else
			{
				Deployment.Current.Dispatcher.BeginInvoke(() =>
				{
					AssemblyPart part = new AssemblyPart();
					Assembly loadedAssembly = part.Load(resultStram);
					ApplicationCache.SaveAssemblyInCache(loadedAssembly);
				});
			}
#endif
		}

		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1806:DoNotIgnoreMethodResults", MessageId = "System.Windows.Resources.StreamResourceInfo")]
		private static Stream GetSpecificResourceFromStream(Stream stream, Uri uri)
		{
			StreamResourceInfo resource = new StreamResourceInfo(stream, "application / binary");
#if WPF
			StreamResourceInfo resourceDeploymants = Application.GetResourceStream(uri);
#else
			StreamResourceInfo resourceDeploymants = Application.GetResourceStream(resource, uri);
#endif
			return resourceDeploymants.Stream;
		}

		public static void BeginDownloadingXAPs(IList<ExampleItem> examples)
		{
			BackgroundWorker worker = new BackgroundWorker();
			worker.DoWork += OnDoWork;
			worker.RunWorkerAsync(examples);
		}

		private static void OnDoWork(object sender, DoWorkEventArgs e)
		{
			var examples = e.Argument as ExamplesCollection;

			for (int i = 1; i < examples.Count; i++)
			{
				//string xapName = string.Format("Examples.{0}{1}", examples[i].Text, ApplicationAssemblyType);
				LoadAssemblies(string.Empty, string.Empty, null);
			}
		}
	}
}
