﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Threading;
using System.Windows.Data;
using System.ComponentModel;

namespace Telerik.Windows.QuickStart
{
#if !WPF
	using FlowDocument = TextBlock;
	using FlowDocumentScrollViewer = ScrollViewer;
#endif

	public class BaseCodeViewer : Control
	{
		public static readonly DependencyProperty DescriptionProperty =
			DependencyProperty.Register("Description",
										typeof(KeyValuePair<string, string>),
										typeof(BaseCodeViewer),
										new PropertyMetadata(OnExampleDescriptionPropertyChanged));


		public static readonly DependencyProperty SelectionVisibilityProperty =
			DependencyProperty.Register("SelectionVisibility",
										typeof(Visibility),
										typeof(BaseCodeViewer),
										new PropertyMetadata(Visibility.Collapsed));


		protected FlowDocument FormattedText { get; set; }
		protected FlowDocumentScrollViewer FormattedTextScrollViewer { get; set; }
#if !WPF
		protected TextBox Selection { get; set; }
#endif

		public BaseCodeViewer()
		{
			this.DefaultStyleKey = typeof(BaseCodeViewer);
		}

		public override void OnApplyTemplate()
		{
			this.InitializeFlowDocument();

			base.OnApplyTemplate();
		}

		private void InitializeFlowDocument()
		{
			this.FormattedTextScrollViewer = this.GetTemplateChild("ScrollViewer") as FlowDocumentScrollViewer;
			this.FormattedText = this.GetTemplateChild("FormattedText") as FlowDocument;

			this.ResetCodeViewer();
#if !WPF
			this.Selection = (TextBox)this.GetTemplateChild("Selection");
			this.Selection.KeyDown += (sender, e) => { e.Handled = true; };
#endif
		}

		private static void OnExampleDescriptionPropertyChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
		{
			sender.Dispatcher.BeginInvoke(new Action(() =>
				{
					var viewer = sender as BaseCodeViewer;

					var result = e.NewValue as KeyValuePair<string, string>?;

					if (result.HasValue && result.Value.Key != null && result.Value.Value != null)
					{
						viewer.SelectionVisibility = result.Value.Key.GetSourceFileExtension() == ".txt"
							? Visibility.Collapsed
							: Visibility.Visible;

						viewer.ResetCodeViewer();
					}
				}));
		}

		private void ResetCodeViewer()
		{
			if (this.FormattedText != null && Description.Key != null)
			{
				this.ResetCodeViewerContent()
					  .ConvertTabsToSpaces(this.Description.Value, 4)
					  .FormatCodeFile(Description.Value, Description.Key.GetSourceFileExtension());
				this.RefreshTextArea();
			}
		}

		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
		protected void RefreshTextArea()
		{
#if !WPF
			this.Selection.Text = this.FormattedText.Text;
			this.FormattedTextScrollViewer.ScrollToVerticalOffset(0);
			this.FormattedTextScrollViewer.ScrollToHorizontalOffset(0);
#endif
		}

		protected InlineCollection ResetCodeViewerContent()
		{
#if WPF
			Paragraph paragraph = new Paragraph();
			if (this.FormattedText != null)
			{
				this.FormattedText.Blocks.Clear();
				this.FormattedText.Blocks.Add(paragraph);
			}
#else
			TextBlock paragraph = this.FormattedText;
			paragraph.Inlines.Clear();
#endif
			return paragraph.Inlines;
		}

		public KeyValuePair<string, string> Description
		{
			get { return (KeyValuePair<string, string>)GetValue(DescriptionProperty); }
			set { SetValue(DescriptionProperty, value); }
		}

		public Visibility SelectionVisibility
		{
			get { return (Visibility)GetValue(SelectionVisibilityProperty); }
			set { SetValue(SelectionVisibilityProperty, value); }
		}
	}
}
