﻿using System;
using System.Collections.Generic;
using System.Windows;
using Telerik.Windows.Controls;

namespace Telerik.Windows.QuickStart
{
	public class CodeViewer : BaseCodeViewer, ICodeViewer
	{
		internal RadTabControl TabControl { get; set; }

		public static readonly DependencyProperty CodeViewProperty =
			DependencyProperty.Register("CodeView", typeof(string), typeof(CodeViewer),
			new PropertyMetadata(OnCodeViewPropertyChanged));

		public static readonly DependencyProperty ItemsProperty =
			DependencyProperty.Register("Items",
										typeof(IEnumerable<string>),
										typeof(CodeViewer),
										null);

		public CodeViewer()
		{
			this.DefaultStyleKey = typeof(CodeViewer);
		}

		public override void OnApplyTemplate()
		{
			base.OnApplyTemplate();

			this.InitializeTabControl();
		}

		private void InitializeTabControl()
		{
			if (this.TabControl != null)
			{
				this.TabControl.SelectionChanged -= this.OnTabControlSelectionChanged;
			}
			this.TabControl = this.GetTemplateChild("TabControl") as RadTabControl;
			if (this.TabControl != null)
			{
				this.TabControl.SelectionChanged += this.OnTabControlSelectionChanged;
			}
		}

		private static void OnCodeViewPropertyChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
		{
			var codeViewer = sender as CodeViewer;
			if (null != codeViewer)
			{
				string selectedItem = codeViewer.TabControl.SelectedItem as string;
				codeViewer.ResetCodeViewerContent()
						  .FormatCodeFile(e.NewValue.ToString(), selectedItem.GetSourceFileExtension());
				codeViewer.RefreshTextArea();
			}
		}

		private void OnTabControlSelectionChanged(object sender, RoutedEventArgs e)
		{
			EventHandler<CodeViewerEventArgs> eh = null;

			lock (this) { eh = SelectionChanged; }
			if (null != this.SelectionChanged)
			{
				this.SelectionChanged(this, new CodeViewerEventArgs(this.TabControl.SelectedItem));
			}
		}

		public IEnumerable<string> Items
		{
			get { return (IEnumerable<string>)GetValue(ItemsProperty); }
			set { SetValue(ItemsProperty, value); }
		}

		public string CodeView
		{
			get { return (string)GetValue(CodeViewProperty); }
			set { SetValue(CodeViewProperty, value); }
		}

		#region ICodeViewer Members

		public event EventHandler<CodeViewerEventArgs> SelectionChanged;

		#endregion
	}

	public class CodeViewerEventArgs : EventArgs
	{
		internal object SelectedItem { get; set; }

		public CodeViewerEventArgs() { }

		public CodeViewerEventArgs(object selectedItem)
			: this()
		{
			this.SelectedItem = selectedItem;
		}
	}

	public interface ICodeViewer
	{
		event EventHandler<CodeViewerEventArgs> SelectionChanged;
	}
}
