﻿using System;
using System.IO;
using System.Collections.Generic;

namespace Telerik.Windows.QuickStart
{
	public abstract class CodeViewerService
	{
		private static CodeViewerService current;

		public static CodeViewerService Current
		{
			get
			{
				if (current == null)
				{
#if WPF
					if (System.Windows.Interop.BrowserInteropHelper.IsBrowserHosted)
					{
						current = new XbapCodeViewerService();
					}
					else
					{
						current = new WpfCodeViewerService();
					}
#else
					current = new SilverlightCodeViewerService();
#endif
				}
				return current;
			}
		}

		public string GetDescription(string examplePath)
		{
			return this.GetResource(this.GetDescriptionPath(examplePath));
		}

		public abstract void GetSourceFileAsync(string filePath, Action<string> callback);

		public abstract void GetSourceFilesListAsync(string examplePath, Action<string> callback);

		protected string GetResource(string path)
		{
			Stream stream = this.GetResourceStream(path);
			if (stream == null)
			{
				return null;
			}

			using (StreamReader reader = new StreamReader(stream))
			{
				return reader.ReadToEnd();
			}

			//return string.Empty;
		}

		protected static string RemoveFolderSeparator(string examplePath)
		{
			if (examplePath.IndexOf('|') == examplePath.Length - 1)
			{
				examplePath = examplePath.Remove(examplePath.IndexOf('|'));
			}
			return examplePath;
		}

		protected abstract string GetDescriptionPath(string examplePath);

		protected abstract Stream GetResourceStream(string path);
	}
}