﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Globalization;

namespace Telerik.Windows.QuickStart
{
	public class WpfCodeViewerService : CodeViewerService
	{
		protected static string DomainBaseDirectory
		{
			get
			{
#if WPF
				return AppDomain.CurrentDomain.BaseDirectory;
#else
				return string.Empty;
#endif
			}
		}

		public override void GetSourceFileAsync(string filePath, Action<string> callback)
		{
			callback(this.GetResource("/" + filePath));
		}

		public override void GetSourceFilesListAsync(string examplePath, Action<string> callback)
		{
			examplePath = RemoveFolderSeparator(examplePath);

			string[] patterns = new string[] { "*.txt", "*.cs", "*.vb", "*.xaml" };
			List<string> directoryFiles = new List<string>();
			string[] fileNames = examplePath.Split(new char[] { '|' }, StringSplitOptions.RemoveEmptyEntries);

			foreach (string file in fileNames)
			{
				string fullPath = Path.Combine(DomainBaseDirectory, file);

				foreach (string pattern in patterns)
				{
					directoryFiles.AddRange(GetFileName(file, Directory.GetFiles(fullPath, pattern)));
				}
				directoryFiles.Remove(file + "/" + "Description.xaml");
			}

			directoryFiles.Sort((x, y) => Comparer<string>.Default.Compare(x, y));

			string examplesFiles = string.Join("|", directoryFiles.ToArray());

			callback(examplesFiles);
		}

		protected override string GetDescriptionPath(string examplePath)
		{
			return examplePath + "/Description.txt";
			//return string.Format(CultureInfo.InvariantCulture, "/{0}/Description.xaml", examplePath);
		}

		protected override Stream GetResourceStream(string path)
		{
			return File.OpenRead(DomainBaseDirectory + path);
		}

		private static IEnumerable<string> GetFileName(string directoryName, string[] fileNameCollection)
		{
			List<string> fileNames = new List<string>();
			foreach (string fileName in fileNameCollection)
			{
				fileNames.Add(directoryName + "/" + Path.GetFileName(fileName));
			}
			return fileNames;
		}
	}
}