﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows.Media;

namespace Telerik.Windows.QuickStart
{
	public class CodeViewerViewModel : NotifyPropertyChangedBase
	{
		private string exampleTitle;
		public string ExampleTitle
		{
			get
			{
				return this.exampleTitle;
			}
			set
			{
				if (value != exampleTitle)
				{
					exampleTitle = value;
					this.OnPropertyChanged("ExampleTitle");
				}
			}
		}

		private ImageSource exampleImage;
		public ImageSource ExampleImage
		{
			get
			{
				return this.exampleImage;
			}
			set
			{
				if (value != this.exampleImage)
				{
					this.exampleImage = value;
				}
			}
		}

		public Dictionary<string, string> SourceCode
		{
			get;
			private set;
		}

		public KeyValuePair<string, string> CodeView
		{
			get
			{
				return this.SourceCode != null 
					? this.SourceCode.First(keyPair => keyPair.Key == this.selectedItem)
					: new KeyValuePair<string, string>();
			}
		}

		private KeyValuePair<string, string> description;
		public KeyValuePair<string, string> Description
		{
			get
			{
				return this.description;
			}
			set
			{
				this.description = value;
				this.OnPropertyChanged("Description");
			}
		}

		private IEnumerable<string> sourceRelatedFiles;
		public IEnumerable<string> SourceRelatedFiles
		{
			get
			{
				return this.sourceRelatedFiles;
			}
			set
			{
				if (value != this.sourceRelatedFiles)
				{
					this.sourceRelatedFiles = value;
					this.OnPropertyChanged("SourceRelatedFiles");
				}
			}
		}

		private string selectedItem;
		public string SelectedItem
		{
			get
			{
				return this.selectedItem;
			}
			set
			{
				if (value != null)
				{
					this.selectedItem = value;

					if (this.SourceCode != null && this.SourceCode.ContainsKey(value))
					{
						this.OnPropertyChanged("SelectedItem");
						this.OnPropertyChanged("CodeView");
					}
				}
			}
		}

		public CodeViewerViewModel(string relatedFilesToExample)
		{
			CodeViewerService.Current.GetSourceFilesListAsync(relatedFilesToExample, this.OnSourceFilesReceived);
		}

		protected void OnSourceFilesReceived(string list)
		{
			ApplicationHelper.Dispatcher.BeginInvoke(new Action(() =>
			{
				var descriptionFile = list.Split('|').Where(file => Path.GetExtension(file) == ".txt");

				string descrFile = descriptionFile.Count() > 0 ? descriptionFile.First() : string.Empty;

				//// Remove the Description file
				var result = list.Split('|').Where(file => Path.GetExtension(file) != ".txt").ToList();

				var files = new List<string>();
				result.Sort(new Comparison<string>(SortDescription));

				result.ForEach(fileName => files.Add(Path.GetFileName(fileName)));
				this.SourceRelatedFiles = files;

				CodeViewerService.Current.GetSourceFileAsync(descrFile,
							code => this.Description = new KeyValuePair<string, string>(descrFile, code));

				this.SourceCode = new Dictionary<string, string>();

				result.ForEach(file =>
					{
						CodeViewerService.Current.GetSourceFileAsync(file, resDictionary =>
							{
								string fileName = Path.GetFileName(file);
								this.SourceCode.Add(fileName, resDictionary);

								if (fileName == this.SelectedItem)
								{
									this.OnPropertyChanged("CodeView");
								}
							});
					});
			}));
		}

		private int SortDescription(string first, string second)
		{
			first = Path.GetFileName(first);
			second = Path.GetFileName(second);

			int result = Comparer<string>.Default.Compare(first, second);
			if (result != 0)
			{
				if (first.Equals("Example.xaml", StringComparison.OrdinalIgnoreCase))
				{
					return -1;
				}
				if (second.Equals("Example.xaml", StringComparison.OrdinalIgnoreCase))
				{
					return 1;
				}
			}
			return result;
		}
	}
}
