﻿using System.Windows;
using System.Windows.Controls;
using System;
using System.Windows.Media.Animation;
#if !WPF
	using ComboBox = Telerik.Windows.Controls.RadComboBox;
using System.Windows.Markup;
using Telerik.Windows.Controls;
#endif

namespace Telerik.Windows.QuickStart
{
	public partial class Example : UserControl
	{
#if !WPF
		public event EventHandler Unloaded = delegate { };
#endif
		public event EventHandler ShowExampleArea = delegate { };
		public event EventHandler HideExampleArea = delegate { };
		public event EventHandler TransitionAnimationCompleted = delegate { };
		public event EventHandler TransitionAnimationStarted = delegate { };

		public static readonly DependencyProperty ThemeComboBoxVisibilityProperty =
			DependencyProperty.Register("ThemeComboBoxVisibility", typeof(Visibility), typeof(Example), new PropertyMetadata(Visibility.Visible));

		public static readonly DependencyProperty TopRightPanelProperty =
			DependencyProperty.Register("TopRightPanel", typeof(Panel), typeof(Example), null);

		public static readonly DependencyProperty BottomRightPanelProperty =
            DependencyProperty.Register("BottomRightPanel", typeof(Panel), typeof(Example), null);

		public static readonly DependencyProperty OutputProperty =
			DependencyProperty.Register("Output", typeof(string), typeof(Example), null);

		public static readonly DependencyProperty DropDownProperty =
			DependencyProperty.Register("DropDown", typeof(ComboBox), typeof(Example), null);

		public Visibility ThemeComboBoxVisibility
		{
			get { return (Visibility)GetValue(ThemeComboBoxVisibilityProperty); }
			set { SetValue(ThemeComboBoxVisibilityProperty, value); }
		}

		public string Output
		{
			get { return (string)GetValue(OutputProperty); }
			set { SetValue(OutputProperty, value); }
		}

		public ComboBox DropDown
		{
			get { return (ComboBox)GetValue(DropDownProperty); }
			set { SetValue(DropDownProperty, value); }
		}

        public Panel TopRightPanel
		{
            get { return (Panel)GetValue(TopRightPanelProperty); }
			set { SetValue(TopRightPanelProperty, value); }
		}

        public Panel BottomRightPanel
		{
            get { return (Panel)GetValue(BottomRightPanelProperty); }
			set { SetValue(BottomRightPanelProperty, value); }
		}

		public Example() 
		{
#if !WPF
			System.Windows.Resources.StreamResourceInfo sri = null;
			try
			{
				Uri source = new Uri("/Telerik.Windows.QuickStart;component/Themes/CommonStyles.xaml", UriKind.Relative);
				sri = Application.GetResourceStream(source);
				if (sri != null && sri.Stream != null)
				{
					System.Collections.Generic.List<string> keys = new System.Collections.Generic.List<string>();

					System.Windows.ResourceDictionary newDict = Telerik.Windows.Controls.ResourceParser.Parse(sri.Stream, true, keys);
					foreach (string key in keys)
					{
						if (this.Resources.Contains(key))
						{
							this.Resources.Remove(key);
						}

						object value = newDict[key];
						if (value != null)
						{
							this.Resources.Add(key, value);
						}
					}
				}
			}
			catch (System.Collections.Generic.KeyNotFoundException)
			{
			}
#endif
		}

		internal void Unload()
		{
			this.OnUnloaded(EventArgs.Empty);
		}

		internal void AnimationCompleted()
		{
			this.OnTransitionAnimationCompleted(EventArgs.Empty);
		}

		internal void StartTransitionAnimation()
		{
			this.OnTransitionAnimationStarted();
		}

		protected virtual void OnTransitionAnimationStarted()
		{
			this.TransitionAnimationStarted(this, EventArgs.Empty);
		}

		protected virtual void OnTransitionAnimationCompleted(EventArgs e)
		{
			this.TransitionAnimationCompleted(this, e);
		}

		protected virtual void OnUnloaded(EventArgs args)
		{
#if !WPF
			this.Unloaded(this, args);
#endif
		}

		internal void OnSwitchCodeArea(bool isCodeAreaView)
		{
			if (isCodeAreaView)
			{
				this.OnHideExampleArea(EventArgs.Empty);
			}
			else
			{
				var popupOpenDelayer = new Storyboard();
				popupOpenDelayer.Duration = TimeSpan.FromSeconds(2);

				EventHandler handler = null;
				handler = (sender, e) =>
				{
					this.OnShowExampleArea(EventArgs.Empty);
					popupOpenDelayer.Completed -= handler;
				};
				popupOpenDelayer.Completed += handler;
				popupOpenDelayer.Begin();
			}
		}

		protected virtual void OnShowExampleArea(EventArgs e)
		{
			this.ShowExampleArea(this, e);
		}

		protected virtual void OnHideExampleArea(EventArgs e)
		{
			this.HideExampleArea(this, EventArgs.Empty);
		}
	}
}
