﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Linq;
using System.Windows;
#if !WPF
using System.Windows.Browser;
#endif
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Xml.Linq;
using System.ComponentModel;
using System.Collections.Specialized;
using System.Windows.Data;

namespace Telerik.Windows.QuickStart
{
	public class ExampleItem : NotifyPropertyChangedBase, IExampleItem
	{
		#region [Private fields]
		private string filter;
		private bool isSelected;
		private bool isExpanded;
		#endregion

		private bool HasVisibleChildren
		{
			get
			{
				foreach (ExampleItem item in this.Children)
				{
					if (item.Visible)
					{
						return true;
					}
				}
				return false;
			}
		}

		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic", Justification = "Used for binding")]
		public string Platform
		{
			get
			{
#if WPF
				return "for WPF";
#else
				return "for Silverlight";
#endif
			}
		}

		private bool visible;

		public bool Visible
		{
			get
			{
				if (this.IsExample)
				{
					bool isMatch = string.IsNullOrEmpty(this.Filter) ||
								this.Text.IndexOf(this.Filter, StringComparison.OrdinalIgnoreCase) > -1 ||
								this.Keywords.IndexOf(this.Filter, StringComparison.OrdinalIgnoreCase) > -1;

					return visible = isMatch;
				}
				else
				{
					return visible = this.HasVisibleChildren;
				}
			}
			set
			{
				if (value != visible)
				{
					visible = value;
				}
			}
		}

		public string Filter
		{
			get
			{
				return this.filter;
			}
			set
			{
				if (this.filter != value)
				{
					this.filter = value;
					OnPropertyChanged("Visible");
				}
			}
		}

		public string ShortTypeName
		{
			get
			{
				return string.Concat(ApplicationHelper.ExampleApplicationRootNamespace, ".", this.Path.Replace('/', '.'), ".Example");
			}
		}

		public ExamplesCollection Children { get; private set; }
		public string Path { get; set; }
		public string Text { get; private set; }
		public bool IsHomepage { get; private set; }
		public bool IsDefault { get; private set; }
		public bool IsExample { get; private set; }
		public string IconUrl { get; private set; }
		public bool IsFeatured { get; private set; }
		public IExampleItem Parent { get; private set; }
		public string CommonFilesFolder { get; private set; }
		public string Keywords { get; private set; }
		public string RelatedExamplesFilePath { get; private set; }

		public bool IsSelected
		{
			get
			{
				return this.isSelected;
			}
			set
			{
				if (this.isSelected != value)
				{
					if (value)
					{
						this.IsExpanded = true;
					}
					this.isSelected = value;
					this.OnPropertyChanged("IsSelected");
				}
			}
		}

		public bool IsExpanded
		{
			get
			{
				return this.isExpanded;
			}
			set
			{
				if (this.isExpanded != value)
				{
					if (value && this.Parent != null)
					{
						this.Parent.IsExpanded = true;
					}
					this.isExpanded = value;
					this.OnPropertyChanged("IsExpanded");
				}
			}
		}

		public ImageSource ExampleImage
		{
			get
			{
#if !WPF
				string imgUri = string.Format(CultureInfo.InvariantCulture, "/{0};component/Images/Products/Big/{1}.png", ApplicationHelper.ExampleApplicationAssemblyName, this.Path.Split('/')[0]);
#else
				string imgUri = string.Format(CultureInfo.InvariantCulture, "/Images/Products/WPF/Big/{0}.png", this.Path.Split('/')[0]);
#endif
				return new BitmapImage(new Uri(imgUri, UriKind.Relative));
			}
		}

		public ImageSource Icon
		{
			get
			{
				return string.IsNullOrEmpty(this.IconUrl) ? null : new BitmapImage(new Uri(this.IconUrl, UriKind.Relative));
			}
		}

		public IExampleItem CategoryFirstLookExample
		{
			get
			{
				foreach (ExampleItem childItem in this.Children)
				{
					if (childItem.Visible)
					{
						return childItem;
					}
				}
				return this;
			}
		}

		public bool HasElements
		{
			get
			{
				return this.Children.Count > 0;
			}
		}

		private static List<IExampleItem> SelectAll(IEnumerable<XElement> elements, IExampleItem parent)
		{
			var items = new List<IExampleItem>();

			elements.ForEach(element =>
			{
				bool visible = GetAttributeValue(element, "visible", true);
				if (visible)
				{
					var exampleItem = new ExampleItem();

					// Compatibility: the old XML syntax uses "name", we want to use "path"
					exampleItem.Path = GetAttributeValue(element, "path", null) ?? GetAttributeValue(element, "name", string.Empty);
					exampleItem.Text = GetAttributeValue(element, "text", string.Empty);
					exampleItem.IconUrl = GetAttributeValue(element, "IconUrl", exampleItem.Path + "/Image.png");
					exampleItem.IsFeatured = GetAttributeValue(element, "featured", false);
					exampleItem.IsDefault = GetAttributeValue(element, "default", false);
					exampleItem.IsExample = element.Name.LocalName == "example";
					exampleItem.CommonFilesFolder = GetAttributeValue(element, "commonFiles", string.Empty);
					exampleItem.RelatedExamplesFilePath = string.Join("|", new string[] { exampleItem.Path, exampleItem.CommonFilesFolder });
					exampleItem.IsHomepage = GetIsHomeExample(element, exampleItem);
					exampleItem.Parent = parent;
					exampleItem.Keywords = SelectKeyWards(exampleItem, element);
					exampleItem.Children = new ExamplesCollection(SelectAll(element.Elements(), exampleItem));

					items.Add(exampleItem);
				}
			});
			return items;
		}

		private static string SelectKeyWards(IExampleItem currentItem, XElement element)
		{
			string keyWards = GetAttributeValue(element, "KeyWords", string.Empty);
			if (currentItem.Parent != null)
			{
				currentItem = currentItem.Parent;
				keyWards = string.Format("{0},{1},{2}", keyWards, currentItem.Keywords, currentItem.Text);
			}
			return keyWards;
		}

		private static bool GetIsHomeExample(XElement element, IExampleItem exampleItem)
		{
			while (element != null && exampleItem != null && exampleItem.IsExample)
			{
				element = element.Parent;
				exampleItem = exampleItem.Parent;
			}
			return GetAttributeValue(element, "IsHomeExample", false);
		}

		private static string GetAttributeValue(XElement element, string name, string defaultValue)
		{
			return element.Attribute(name) == null ? defaultValue : element.Attribute(name).Value;
		}

		private static bool GetAttributeValue(XElement element, string name, bool defaultValue)
		{
			return element.Attribute(name) == null ? defaultValue : Convert.ToBoolean(element.Attribute(name).Value, CultureInfo.InvariantCulture);
		}

		internal void ApplyFilter(string value)
		{
			this.Filter = value;

			foreach (ExampleItem item in this.Children)
			{
				item.ApplyFilter(value.Trim());
			}
			this.OnPropertyChanged("Visible");
		}

		public static ExamplesCollection SelectAll(IEnumerable<XElement> elements)
		{
			var items = SelectAll(elements, null);

			items.Sort((example1, example2) =>
				{
					int result = Comparer<string>.Default.Compare(example1.Text, example2.Text);
					if (result != 0)
					{
						if (example1.Text.Equals("radcontrols", StringComparison.OrdinalIgnoreCase))
						{
							return -1;
						}
						if (example2.Text.Equals("radcontrols", StringComparison.OrdinalIgnoreCase))
						{
							return 1;
						}
					}
					return result;
				});

			return new ExamplesCollection(items);
		}
	}
}
