﻿using System.Windows;
using System.Windows.Controls;
using Telerik.Windows.Controls;

namespace Telerik.Windows.QuickStart
{
    public partial class BasePage : UserControl, IFrame
    {
		public BasePage()
		{
#if WPF
			this.SetValue(Page.KeepAliveProperty, false);
#endif
		}

        private string hash;

        #region [ DependancyProperties ]
        public static readonly DependencyProperty TitleProperty =
           DependencyProperty.Register("Title", typeof(string), typeof(ExamplePage), null);

        internal static readonly DependencyProperty SelectedtExampleProperty =
            DependencyProperty.Register("SelectedExample", 
                                         typeof(Example), 
                                         typeof(BasePage),
                                         new PropertyMetadata(OnItemChanged));

        public static readonly DependencyProperty ExampleItemProperty =
            DependencyProperty.Register("ExampleItem",
										typeof(IExampleItem), 
                                        typeof(BasePage), 
                                        new PropertyMetadata(ExampleItemChanged));

        internal Example SelectedExample
        {
            get { return (Example)GetValue(SelectedtExampleProperty); }
            set { SetValue(SelectedtExampleProperty, value); }
        }

		public IExampleItem ExampleItem
        {
			get { return (IExampleItem)GetValue(ExampleItemProperty); }
            set { SetValue(ExampleItemProperty, value); }
        }

        public string Title
        {
            get { return (string)GetValue(TitleProperty); }
            set { SetValue(TitleProperty, value); }
        }

        #endregion

        internal static BasePage GetValidPage(IExampleItem exampleItem)
        {
            if (exampleItem.IsHomepage)
            {
				return new Homepage();
            }
            else
            {
				return new ExamplePage();
            }
        }

        private static void OnItemChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            (sender as BasePage).OnCurrentExampleChanged(e.OldValue, e.NewValue);
        }

        private static void ExampleItemChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            (d as BasePage).OnExampleItemChanged(e.OldValue, e.NewValue);
        }

        protected virtual void OnExampleItemChanged(object oldItem, object newItem) { }

        protected virtual void OnCurrentExampleChanged(object oldValue, object newValue) { }

        #region IFrame Members

        public string Path
        {
            get
            {
                return this.GetType().FullName;
            }
        }

        public string NavigationIdentifier
        {
            get
            {
                if (string.IsNullOrEmpty(this.hash))
                {
                    return this.Path;
                }
                return this.hash;
            }
            set
            {
                this.hash = value;
            }
        }

        public Panel ParentContainer
        {
            get;
            set;
        }

        #endregion
    }
}
