﻿using System;
using System.Globalization;
using System.Windows;

namespace Telerik.Windows.QuickStart
{
	internal class BrowserHelper
	{
		internal static string PathToExampleXML
		{
			get
			{
				return string.Format(CultureInfo.InvariantCulture, "/{0};component/Examples.xml", ApplicationHelper.ExampleApplicationAssemblyName);
			}
		}

		internal static Uri GetCurrentBrowserUri
		{
			get
			{
#if WPF
				return System.Windows.Interop.BrowserInteropHelper.Source;
#else
				return Application.Current.Host.Source;
#endif
			}
		}

		internal static string GetCurrentBookmark(string defaultValue)
		{
#if WPF
			Uri site = GetCurrentBrowserUri;
			if (site != null)
			{
				string query = site.Query;
				if (!string.IsNullOrEmpty(query))
				{
					return query.Substring(1, query.Length - 1);
				}
			}
			return defaultValue;
#else
			string bookmark = System.Windows.Browser.HtmlPage.Window.CurrentBookmark;
			if (string.IsNullOrEmpty(bookmark))
			{
				// Add support for query string parameters: /Default.aspx?/Category/Example
				bookmark = System.Windows.Browser.HtmlPage.Document.DocumentUri.Query;
				if (string.IsNullOrEmpty(bookmark))
				{
					return defaultValue;
				}
				// Remove the ? from the beginning of the query string
				bookmark = bookmark.Substring(1);
			}
			return bookmark;
#endif
		}

		internal static void SetBrowserTitle(string title)
		{
#if !WPF
			string htmlTitle = string.Format(CultureInfo.InvariantCulture, "window.document.title=\'{0}\'", title);
			
			System.Windows.Browser.HtmlPage.Window.Eval(htmlTitle);
#else
			Application.Current.MainWindow.Title = title;
#endif
		}
	}
}
