﻿using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Media;
using System.Windows.Shapes;
using Telerik.Windows.Controls;
using Telerik.Windows.QuickStart.Animation;
using Telerik.Windows.QuickStart.Parser;
#if WPF
using RadFrame = System.Windows.Controls.Frame;
#endif

namespace Telerik.Windows.QuickStart
{
	public static class ExtensionMethods
	{
		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1011:ConsiderPassingBaseTypesAsParameters", Justification = "WPF compatibility")]
		public static void NavigateToExample(this RadFrame exampleContainer, bool switchOffAnimation, BasePage page)
		{
#if WPF
			ExamplePage currentPage = exampleContainer.NavigationService.Content as ExamplePage;

			NavigationAnimator navigation = new NavigationAnimator();
			navigation.Navigate(exampleContainer, page);
			navigation.TransitionAnimationCompleted += OnTransitionAnimationCompleted;
#else
			NavigationService service = NavigationService.GetNavigationService();

			var mask1 = new Rectangle()
			{
				Fill = new SolidColorBrush(Color.FromArgb(77, 8, 17, 48))
			};

			var mask2 = new Rectangle()
			{
				Fill = new SolidColorBrush(Color.FromArgb(77, 8, 17, 48))
			};
			var grid = service.RootPanel as Grid;

			var currentPage = service.RootPanel.Children[0] as BasePage;
			ExampleTransition exampleTransition = new ExampleTransition(grid.ActualWidth, grid, mask1, mask2);
			if (!switchOffAnimation) service.Transition = exampleTransition;
			exampleTransition.TransitionAnimationCompleted += OnTransitionAnimationCompleted;

			exampleContainer.Navigate(page, service.Transition);
#endif
			if (currentPage != null)
			{
				currentPage.SelectedExample.Unload();
				currentPage.SelectedExample.StartTransitionAnimation();
			}
		}

		private static void OnTransitionAnimationCompleted(object sender, TransitionEventArgs e)
		{
			var transition = sender as ExampleTransition;
			transition.TransitionAnimationCompleted -= OnTransitionAnimationCompleted;

			if (e.TargetExamplePage != null && e.TargetExamplePage is BasePage)
			{
				var examplePage = e.TargetExamplePage as BasePage;
				examplePage.SelectedExample.AnimationCompleted();
			}
		}

		public static bool Contains(this Dictionary<string, object> source, string key)
		{
			return source.ContainsKey(key);
		}
	}

	public static class IEnumerableExtension
	{
		public static void ForEach<TSource>(this IEnumerable<TSource> collection, Action<TSource> action)
		{
			foreach (var item in collection)
			{
				action(item);
			}
		}

		public static IEnumerable<TSource> ConvertTabsToSpaces<TSource>(this IEnumerable<TSource> collection, string sourceText, int numberOfSpaces)
		{
			Regex.Replace(sourceText, "\t", string.Empty.PadLeft(numberOfSpaces, ' '));
			return collection;
		}

		public static IEnumerable<TSource> FormatCodeFile<TSource>(this IEnumerable<TSource> collection, string code, string extension)
		{
			List<Token> tokens = new Tokenizer().TokenizeCode(code, extension);

			var coll = collection as InlineCollection;
			if (null != coll)
			{
				tokens.ForEach(token => coll.Add(token.GetRun()));
			}
			return collection;
		}
	}

	public static class StringExtension
	{
		public static string RemoveWhiteSpace(this string value)
		{
			int index;
			while ((index = value.IndexOf(' ')) != -1)
			{
				value = value.Remove(index, 1);
			}
			return value;
		}

		public static string GetSourceFileExtension(this string value)
		{
			string extension = System.IO.Path.GetExtension(value);
			if (string.IsNullOrEmpty(extension))
			{
				return ".txt";
			}
			return extension;
		}
	}
}
