﻿using System;
using System.Windows.Controls;
using System.Windows;
using System.Windows.Documents;
using System.Windows.Interop;
using System.Diagnostics;

namespace Telerik.Windows.QuickStart
{
	public class LinkControl : ContentControl
	{
		public static readonly DependencyProperty NavigateUriProperty =
			DependencyProperty.Register("NavigateUri", typeof(string), typeof(LinkControl), null);

		public static readonly DependencyProperty IsPopupProperty =
			DependencyProperty.Register("IsPopup", typeof(bool), typeof(LinkControl), new PropertyMetadata(false));

		private Hyperlink hyperLink;

		public LinkControl()
		{
			this.DefaultStyleKey = typeof(LinkControl);
		}

		public override void OnApplyTemplate()
		{
			base.OnApplyTemplate();

			hyperLink = this.GetTemplateChild("hyperLink") as Hyperlink;

			if (hyperLink != null)
			{
				if (!BrowserInteropHelper.IsBrowserHosted)
				{
					if (!string.IsNullOrEmpty(NavigateUri))
					{
						if (Uri.IsWellFormedUriString(NavigateUri, UriKind.Absolute))
						{
							hyperLink.Click += (sender, e) => { Process.Start(NavigateUri); };
						}
					}
				}
				else
				{
					hyperLink.NavigateUri = new Uri(NavigateUri, UriKind.RelativeOrAbsolute);
					hyperLink.TargetName = IsPopup ? "_blank" : "_self";
				}
			}
		}

		public string NavigateUri
		{
			get { return (string)GetValue(NavigateUriProperty); }
			set { SetValue(NavigateUriProperty, value); }
		}

		public bool IsPopup
		{
			get { return (bool)GetValue(IsPopupProperty); }
			set { SetValue(IsPopupProperty, value); }
		}
	}
}
