﻿using System;
using System.Windows.Controls;
using System.Windows.Input;
using Telerik.Windows.Controls;

namespace Telerik.Windows.QuickStart
{
#if WPF
	using RadFrame = System.Windows.Controls.Frame;
#else
	using Page = System.Windows.Controls.Control;
	using SelectionChangedEventArgs = Telerik.Windows.Controls.SelectionChangedEventArgs;
	using System.Windows;
#endif

	public partial class MainPage : Page
	{
		#region [ Properties ]
		private RadFrame exampleContainer;
		private WatermarkedTextBox searchTextBox;
		private RadTreeView examplesTree;
		private Button searchButton;

		private RadTreeView ExamplesTree
		{
			get
			{
				return this.examplesTree;
			}
			set
			{
				if (this.examplesTree != null)
				{
					this.examplesTree.SelectionChanged -= this.OnExamplesTreeSelectionChanged;
				}
				this.examplesTree = value;
				this.examplesTree.SelectionChanged += this.OnExamplesTreeSelectionChanged;
			}
		}
		private WatermarkedTextBox SearchTextBox
		{
			get
			{
				return this.searchTextBox;
			}
			set
			{
				if (null != this.searchTextBox)
				{
					this.searchTextBox.WatermarkedTextChanged -= this.OnSearchTextChanged;
					this.searchTextBox.KeyDown -= this.OnSearchTextKeyDown;
				}
				this.searchTextBox = value;
				this.searchTextBox.WatermarkedTextChanged += this.OnSearchTextChanged;
				this.searchTextBox.KeyDown += this.OnSearchTextKeyDown;
			}
		}
		private Button SearchButton
		{
			get
			{
				return this.searchButton;
			}
			set
			{
				this.searchButton = value;
				searchButton.Click += (sender, e) => this.SearchForExample();
			}
		}

		internal ExamplesViewModel ViewModel { get; set; }
		public bool AsyncDownloadExamples { get; set; }
		public bool SwitchOffAnimation { get; set; }
		#endregion

		public MainPage()
		{
			this.DefaultStyleKey = typeof(MainPage);

			this.DataContext = this.ViewModel = new ExamplesViewModel();
		}

		public override void OnApplyTemplate()
		{
			base.OnApplyTemplate();

			this.exampleContainer = (RadFrame)this.GetTemplateChild("ExampleContainer");
			this.ExamplesTree = (RadTreeView)this.GetTemplateChild("ExamplesTree");
			this.SearchButton = (Button)this.GetTemplateChild("searchButton");
			this.SearchTextBox = (WatermarkedTextBox)this.GetTemplateChild("searchTextBox");

			this.ApplyMainPageTemplate();

			string exampleToSelect = BrowserHelper.GetCurrentBookmark(string.Empty);
			if (string.IsNullOrEmpty(exampleToSelect))
			{
				this.ViewModel.SelectFirstVisibleExample();
			}
			else
			{
				this.ViewModel.FindExampleBypath(exampleToSelect);
			}
			AddToBrowserHistory(this.ViewModel.SelectedExample);
		}

		private void OnSearchTextKeyDown(object sender, KeyEventArgs e)
		{
			if (e.Key == Key.Enter)
			{
				SearchForExample();
			}
		}

		private void SearchForExample()
		{
			this.ViewModel.SelectFirstVisibleExample();
		}

		internal void Navigate(IExampleItem item)
		{
			if (item.IsExample)
			{
				string searchType = item.ShortTypeName;
#if !WPF
				this.LoadingVisual.Begin();
#endif
				ApplicationHelper.LoadAssemblies(item.Path, searchType, result =>
				{
					this.NavigateToExample(result as Example, item);
#if !WPF
					this.LoadingVisual.Stop();
#endif
					item.IsSelected = true;

				});
			}
			BrowserHelper.SetBrowserTitle(this.ViewModel.BrowserTitle);
		}

		private void NavigateToExample(Example example, IExampleItem item)
		{
			var page = BasePage.GetValidPage(item);

			page.SelectedExample = example;
			page.ExampleItem = item;

			this.exampleContainer.NavigateToExample(this.SwitchOffAnimation, page);
		}

		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
		private void AddToBrowserHistory(IExampleItem item)
		{
#if !WPF
			BrowserHistoryManager.AddBrowserHistory(item.Path, string.Empty);
#else
			Navigate(item);
#endif
		}

		private void OnSearchTextChanged(object sender, EventArgs e)
		{
			this.ViewModel.SearchText = this.searchTextBox.Text;
		}

		private void OnBrowserHistoryNavigateCompleted()
		{
			if (this.ViewModel.SelectedExample != null)
			{
				this.Navigate(this.ViewModel.SelectedExample);
			}
		}

		private void OnExamplesTreeSelectionChanged(object sender, SelectionChangedEventArgs e)
		{
			var example = this.ExamplesTree.SelectedItem as ExampleItem;

			if (example != null && example != this.ViewModel.SelectedExample)
			{
				this.ViewModel.SelectedExample = example;

				this.AddToBrowserHistory(this.ViewModel.SelectedExample);
			}
		}
	}
}
