﻿using System;
using System.Windows;
using System.Windows.Controls;
using Telerik.Windows.Controls;
using Telerik.Windows.QuickStart.Animation;

namespace Telerik.Windows.QuickStart
{
#if !WPF
	using Page = System.Windows.Controls.Control;
	
	public partial class MainPage : Page
	{
		private LoadingSpin LoadingVisual;
		private FrameworkElement splashLoader;
		private FrameworkElement topLeftLogo;
		private FrameworkElement layoutRoot;
		private FrameworkElement topRightButtons;
		private FrameworkElement topRightPanel;
		private FrameworkElement searchBox;
		private FrameworkElement treeViewPanel;
		private FrameworkElement examplesPanel;
		private FrameworkElement examplesMask;

		internal void ApplyMainPageTemplate()
		{
			this.LoadingVisual = (LoadingSpin)this.GetTemplateChild("LoadingVisual");

			splashLoader = this.GetTemplateChild("splashLoader") as FrameworkElement;
			topLeftLogo = this.GetTemplateChild("topLeftLogo") as FrameworkElement;
			layoutRoot = this.GetTemplateChild("layoutRoot") as FrameworkElement;
			//Top right:
			topRightButtons = this.GetTemplateChild("topRightButtons") as FrameworkElement;
			topRightPanel = this.GetTemplateChild("topRightPanel") as FrameworkElement;
			searchBox = this.GetTemplateChild("searchBox") as FrameworkElement;
			treeViewPanel = this.GetTemplateChild("treeViewPanel") as FrameworkElement;
			examplesPanel = this.GetTemplateChild("examplesPanel") as FrameworkElement;
			examplesMask = this.GetTemplateChild("examplesMask") as FrameworkElement;

			NavigationService service = NavigationService.GetNavigationService();
			service.RootPanel = (RadFrameContainer)this.GetTemplateChild("radFameContainer");

			BrowserHistoryManager.BrowserHistoryNavigate += (sender, e) => { OnBrowserHistoryNavigateCompleted(); };

			this.Dispatcher.BeginInvoke(() =>
			{
				layoutRoot.Opacity = 1;
				splashLoader.Opacity = 1;

				if (!this.SwitchOffAnimation)
				{
					BeginInitialAnimation();
				}
				else
				{
					splashLoader.Opacity = 0;
					examplesMask.Opacity = 0;
				}
			});

			//Add cliping:
			(exampleContainer.Parent as Panel).SizeChanged += (sender, e) =>
			{
				var panel = sender as Panel;
				panel.Clip = new System.Windows.Media.RectangleGeometry()
				{
					Rect = new Rect(new Point(), e.NewSize)
				};
			};
		}

		private void BeginInitialAnimation()
		{
			var logoWidth = topLeftLogo.ActualWidth;
			
			var treeWidth = treeViewPanel.Width;
			var exampleWidth = examplesPanel.ActualWidth;
			var buttonsHeight = topRightButtons.ActualHeight;

			var scaleUpSpline = new Spline(0.321000009775162, 0, 0.45100000500679, 1);

			var initialAnimation = splashLoader
						.Animate()
							.Opacity(0, 1, 1, 0)
						.Animate(topLeftLogo)
							.EnsureDefaultTransforms()
							.Opacity(0, 0, 1, 0, 2, 1)
							.MoveX(0, logoWidth, 1, logoWidth, 2, 0)
								.Splines(2, scaleUpSpline)
						.Animate(treeViewPanel, searchBox)
							.EnsureDefaultTransforms()
							.Opacity(0, 0, 2, 0, 3, 1)
							.MoveX(0, -logoWidth, 2, -logoWidth, 3, 0)
								.Splines(2, scaleUpSpline)
						.Animate(examplesMask)
							.Opacity(0, 1, 5, 1, 5.5, 0)
						.Animate(examplesPanel)
							.EnsureDefaultTransforms()
							.Origin(0, 0.5)
							.Scale(0, 0.8, 5, 0.8, 5.5, 1)
								.Splines(2, scaleUpSpline)
							.MoveX(0, exampleWidth, 3, exampleWidth, 5, 0)
								.Splines(2, scaleUpSpline)
						.Animate(topRightButtons, topRightPanel)
							.EnsureDefaultTransforms()
							.MoveY(0, -buttonsHeight, 5.5, -buttonsHeight, 6, 0)
								.Splines(2, scaleUpSpline)
						.Instance;

			initialAnimation.SpeedRatio = 1.5;

			initialAnimation.Completed += OnInitialAnimationCompleted;

			initialAnimation.Begin();
		}

		private void OnInitialAnimationCompleted(object sender, EventArgs args)
		{
			var service = NavigationService.GetNavigationService();

			if (service.RootPanel != null && service.RootPanel.Children.Count > 0)
			{
				var examplePage = service.RootPanel.Children[0] as BasePage;
				if (examplePage != null)
				{
					examplePage.SelectedExample.AnimationCompleted();
				}
			}
		}
	}
#endif
}
