﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Windows.Media;
using System.Xml;

namespace Telerik.Windows.QuickStart.Parser
{
	public enum HtmlTextDecoration
	{
		/// <summary>
		/// 
		/// </summary>
		Bold,

		/// <summary>
		/// 
		/// </summary>
		Italic,

		/// <summary>
		/// 
		/// </summary>
		None
	}

	internal class HTMLSyntaxParser : SyntaxParser
	{
		public static Brush GetSpecificBrush(HtmlTextDecoration textDecoration)
		{
			switch (textDecoration)
			{
				case HtmlTextDecoration.Bold:
				case HtmlTextDecoration.Italic:
#if !WPF
					return new SolidColorBrush((Color)ColorConverter.ConvertFromString("#ff4fbfe4"));
#else
					return new SolidColorBrush((Color)ColorConverter.ConvertFromString("#FFBE86F4"));
#endif
				default:
					return new SolidColorBrush(Colors.Black);
			}
		}

		public static IEnumerable<Token> ApplyStyle(string htmlValue)
		{
			Collection<Token> tokens = new Collection<Token>();

			using (var stringReader = new StringReader(string.Concat("<xml>", htmlValue, "</xml>")))
			{
				bool isBold = false;
				bool isItalic = false;

				using (var xmlReader = XmlReader.Create(stringReader))
				{
					while (xmlReader.Read())
					{
						string name = xmlReader.Name.ToLower();
						switch (xmlReader.NodeType)
						{
							case XmlNodeType.Element:
							case XmlNodeType.EndElement:
								{
									switch (name.ToLower())
									{
										case "bold":
										case "strong":
										case "b":
											isBold = !isBold;
											break;
										case "i": isItalic = !isItalic;
											break;
									}
									break;
								}

							case XmlNodeType.Text:
								{
									Brush brush = GetSpecificBrush(HtmlTextDecoration.Bold);
									tokens.Add(new Token(xmlReader.Value, brush, isBold, isItalic));
									break;
								}
							case XmlNodeType.Whitespace:
								tokens.Add(new Token(" ", GetSpecificBrush(HtmlTextDecoration.None)));
								break;
						}
					}
				}
			}

			return tokens;
		}

		protected override Collection<LanguageSyntaxStructure> LoadLanguageSyntax(Collection<LanguageSyntaxStructure> languageSyntax)
		{
			//// Load breaks parser
			string htmlPattern = @"<br \>";
			languageSyntax.Add(new LanguageSyntaxStructure(htmlPattern, "Breaks", new SolidColorBrush(Colors.Red)));

			////Load bold parser
			string boldPattern = @"<\s*(b|B|strong|STRONG)\s*>.*<\s*/(b|B|strong|STRONG)\s*>";

			////@"<\s*/?b\s*>";
			languageSyntax.Add(new LanguageSyntaxStructure(boldPattern, "Bold", new SolidColorBrush(Color.FromArgb(255, 0, 0, 100))));

			////Load itelic parser
			string italicPattern = @"(?<Italic><\s*(i|I)\s*>.*<\s*/(|i|I)\s*>)";
			languageSyntax.Add(new LanguageSyntaxStructure(italicPattern, "Italic", new SolidColorBrush(Colors.Green)));

			return base.LoadLanguageSyntax(languageSyntax);
		}

		protected override void SetFileExtension(string extension)
		{
			base.SetFileExtension(".txt");
		}
	}
}
