﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;

namespace Telerik.Windows.QuickStart
{
    /// <summary>
    /// Direction  enum.
    /// </summary>
    public enum Direction
    {
        /// <summary>
        /// Slide UP.
        /// </summary>
        Up = 0,

        /// <summary>
        /// Slide Down.
        /// </summary>
        Down = 1,

        /// <summary>
        /// Slide Right.
        /// </summary>
        Right = 2,

        /// <summary>
        /// Slide Left.
        /// </summary>
        Left = 3
    }

    /// <summary>
    /// SlideTransition class.
    /// </summary>
    public class PageSlideTransition : ITransition
    {
        private object target;
        private TimeSpan timeDuration;
        private Control newPage;
        private Control currentPage;
        private Storyboard currentAnimation;

        /// <summary>
        /// Initializes a new instance of the <see cref="SlideTransition"/> class.
        /// </summary>
        /// <param name="target">The target.</param>
        /// <param name="duration">The duration.</param>
        /// <param name="direction">The direction.</param>
        public PageSlideTransition(Panel target, TimeSpan duration, Direction direction)
        {
            this.Target = target;
            this.timeDuration = duration;
            this.IsRunning = false;
            this.AnimationMode = direction;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SlideTransition"/> class.
        /// </summary>
        /// <param name="target">The target.</param>
        /// <param name="duration">The duration.</param>
        public PageSlideTransition(Panel target, TimeSpan duration)
        {
            this.Target = target;
            this.timeDuration = duration;
            this.IsRunning = false;
            this.AnimationMode = Direction.Down;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SlideTransition"/> class.
        /// </summary>
        public PageSlideTransition()
        {
        }

        /// <summary>
        /// Gets or sets the target.
        /// </summary>
        /// <value>The target.</value>
        public object Target
        {
            get
            {
                return this.target;
            }
            set
            {
                if (value is Panel)
                {
                    this.target = value;
                }
                else
                {
                    throw new NotImplementedException();
                }
            }
        }

        /// <summary>
        /// Gets or sets the current story board.
        /// </summary>
        /// <value>The current story board.</value>
        public Storyboard CurrentStoryboard
        {
            get
            {
                return this.currentAnimation;
            }
            set
            {
                this.currentAnimation = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is running.
        /// </summary>
        /// <value>
        /// 	<c>True</c> if this instance is running; otherwise, <c>false</c>.
        /// </value>
        public bool IsRunning
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the slide mode.
        /// </summary>
        /// <value>The slide mode.</value>
        public Direction AnimationMode
        {
            get;
            set;
        }

		/// <summary>
		/// Stops this instance.
		/// </summary>
        public void StopAnimation()
        {
            if (this.CurrentStoryboard != null)
            {
                this.CurrentStoryboard.Stop();
            }
        }

        /// <summary>
        /// Transitions the page.
        /// </summary>
        /// <param name="previousFrame">The previous frame.</param>
        /// <param name="nextFrame">The next frame.</param>
        public void PerformFrameTransition(IFrame previousFrame, IFrame nextFrame)
        {
            this.IsRunning = true;
            this.newPage = nextFrame as Control;
            this.currentPage = previousFrame as Control;
            StackPanel panel = ((StackPanel)this.Target);
            panel.Children.Clear();
            panel.Children.Add(this.newPage);

            if ((this.currentPage != null) && (this.currentPage != this.newPage))
            {
                System.Windows.Application targetApp = System.Windows.Application.Current;
                if ((((previousFrame.ParentFrame as RadFrameContainer) != targetApp.RootVisual) && (panel != targetApp.RootVisual)) ||
                    (((previousFrame.ParentFrame as RadFrameContainer) == targetApp.RootVisual) && (panel == targetApp.RootVisual)))
                {
                    this.currentPage.RenderTransform = null;
                    if (this.AnimationMode == Direction.Right || this.AnimationMode == Direction.Left)
                    {
                        panel.Orientation = Orientation.Horizontal;
                    }
                    else
                    {
                        panel.Orientation = Orientation.Vertical;
                    }
                    panel.Children.Add(this.currentPage);
                    panel.UpdateLayout();

                    //panel.Height = currentPage.ActualHeight;
                    Storyboard sb = this.GetStoryboard(((Panel)this.Target));
                    this.CurrentStoryboard = sb;
                    if (this.newPage.ActualHeight == 0.0)
                    {
                        panel.Height = this.currentPage.ActualHeight * 2;
                    }
                    panel.Clip =
                             new RectangleGeometry()
                             {
                                 Rect = new Rect(0, 0, targetApp.RootVisual.DesiredSize.Width, targetApp.RootVisual.DesiredSize.Height)
                             };
                    sb.Completed += new EventHandler(this.Sb_Completed);
                    sb.Begin();
                }
            }
        }

        private void Sb_Completed(object sender, EventArgs e)
        {
            Panel panel = ((Panel)this.Target);
            if (panel.Children.Count == 2)
            {
                panel.Children.Remove(this.currentPage);
            }
            panel.UpdateLayout();
            this.IsRunning = false;
        }

        private Storyboard GetStoryboard(Panel container)
        {
            double height = container.ActualHeight;
            double width = container.ActualWidth;

            TranslateTransform topTransform = null;
            TranslateTransform bottomTransform = null;
            Transform existingTransform = null;

            FrameworkElement topContent = (FrameworkElement)container.Children[1];
            FrameworkElement bottomContent = (FrameworkElement)container.Children[0];

            existingTransform = topContent.RenderTransform;
            if (existingTransform != null)
            {
                topTransform = existingTransform as TranslateTransform;

                if (topTransform == null)
                {
                    TransformGroup transformGroup = existingTransform as TransformGroup;

                    if (transformGroup != null)
                    {
                        foreach (Transform transform in transformGroup.Children)
                        {
                            topTransform = transform as TranslateTransform;
                            if (topTransform != null)
                            {
                                break;
                            }
                        }
                        if (topTransform == null)
                        {
                            topTransform = new TranslateTransform();
                            transformGroup.Children.Add(topTransform);
                        }
                    }
                }
            }
            existingTransform = bottomContent.RenderTransform;
            if (existingTransform != null)
            {
                bottomTransform = existingTransform as TranslateTransform;

                if (bottomTransform == null)
                {
                    TransformGroup transformGroup = existingTransform as TransformGroup;

                    if (transformGroup != null)
                    {
                        foreach (Transform transform in transformGroup.Children)
                        {
                            bottomTransform = transform as TranslateTransform;
                            if (bottomTransform != null)
                            {
                                break;
                            }
                        }
                        if (bottomTransform == null)
                        {
                            bottomTransform = new TranslateTransform();
                            transformGroup.Children.Add(bottomTransform);
                        }
                    }
                }
            }

            if (topTransform == null)
            {
                topTransform = new TranslateTransform();
                topContent.RenderTransform = topTransform;
            }
            if (bottomTransform == null)
            {
                bottomTransform = new TranslateTransform();
                bottomContent.RenderTransform = bottomTransform;
            }
            Storyboard slideAnimation = new Storyboard();
            slideAnimation.Duration = this.timeDuration;
            this.SetDoubleAnimation(slideAnimation, topTransform, bottomTransform, height, width);
            return slideAnimation;
        }

        private void SetDoubleAnimation(Storyboard slideAnimation, TranslateTransform topTransform, TranslateTransform bottomTransform, double height, double width)
        {
            DoubleAnimation top = new DoubleAnimation();
            DoubleAnimation bottom = new DoubleAnimation();

            switch (this.AnimationMode)
            {
                case Direction.Up:
                    topTransform.Y = -height;
                    top.Duration = this.timeDuration;
                    top.From = -height / 2;
                    top.To = -height;

                    bottom.Duration = this.timeDuration;
                    bottom.From = height / 2;
                    bottom.To = 0;
                    bottomTransform.Y = height;
                    slideAnimation.Children.Add(top);
                    slideAnimation.Children.Add(bottom);
                    Storyboard.SetTarget(top, topTransform);
                    Storyboard.SetTargetProperty(top, new PropertyPath("Y"));
                    Storyboard.SetTarget(bottom, bottomTransform);
                    Storyboard.SetTargetProperty(bottom, new PropertyPath("Y"));
                    break;

                case Direction.Down:
                    topTransform.Y = height;
                    top.Duration = this.timeDuration;
                    top.From = -height / 2;
                    top.To = 0;
                    bottom.Duration = this.timeDuration;
                    bottom.From = -height / 2;
                    bottom.To = 0;
                    slideAnimation.Children.Add(top);
                    slideAnimation.Children.Add(bottom);
                    Storyboard.SetTarget(top, topTransform);
                    Storyboard.SetTargetProperty(top, new PropertyPath("Y"));
                    Storyboard.SetTarget(bottom, bottomTransform);
                    Storyboard.SetTargetProperty(bottom, new PropertyPath("Y"));
                    break;
                case Direction.Right:

                    topTransform.X = width;
                    top.Duration = this.timeDuration;
                    top.From = -width / 2;
                    top.To = 0;
                    bottom.Duration = this.timeDuration;
                    bottom.From = -width / 2;
                    bottom.To = 0;
                    slideAnimation.Children.Add(top);
                    slideAnimation.Children.Add(bottom);
                    Storyboard.SetTarget(top, topTransform);
                    Storyboard.SetTargetProperty(top, new PropertyPath("X"));
                    Storyboard.SetTarget(bottom, bottomTransform);
                    Storyboard.SetTargetProperty(bottom, new PropertyPath("X"));
                    break;
                case Direction.Left:

                    topTransform.X = -width;
                    top.Duration = this.timeDuration;
                    top.From = -width / 2;
                    top.To = -width;
                    bottom.Duration = this.timeDuration;
                    bottom.From = width / 2;
                    bottom.To = 0;
                    slideAnimation.Children.Add(top);
                    slideAnimation.Children.Add(bottom);
                    Storyboard.SetTarget(top, topTransform);
                    Storyboard.SetTargetProperty(top, new PropertyPath("X"));
                    Storyboard.SetTarget(bottom, bottomTransform);
                    Storyboard.SetTargetProperty(bottom, new PropertyPath("X"));
                    break;
                default:
                    break;
            }
        }
    }
}
