﻿using System;
using System.Globalization;
using System.Windows;
using System.Windows.Controls;

namespace Telerik.Windows.QuickStart
{
	public class VersionPresenter : Control
	{
		public static readonly DependencyProperty VersionTextProperty;
		public static readonly DependencyProperty LoadedExampleProperty;

		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1810:InitializeReferenceTypeStaticFieldsInline")]
		static VersionPresenter()
		{
			string versionNumber = typeof(Telerik.Windows.Controls.StyleManager).AssemblyQualifiedName.
				Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries)[2].Trim();

			////versionNumber = "Version=2008.3.1301.1020";
			string[] versionParts = versionNumber.Split('.');

			int quarter = Convert.ToInt32(versionParts[1], CultureInfo.InvariantCulture);

			int year = Convert.ToInt32(versionParts[0].Remove(0, "Version=".Length), CultureInfo.InvariantCulture);
			int month;
            int day;
            if (versionParts[2].Length > 3)
            {
                month = Convert.ToInt32(versionParts[2].Substring(0, 2), CultureInfo.InvariantCulture);
                day = Convert.ToInt32(versionParts[2].Substring(2), CultureInfo.InvariantCulture);
            }
            else
            {
                month = Convert.ToInt32(versionParts[2].Substring(0, 1), CultureInfo.InvariantCulture);
                day = Convert.ToInt32(versionParts[2].Substring(1), CultureInfo.InvariantCulture);
            }


			if (month > 12)
			{
				month = month-12;
				year++;
			}

			int frameworkVersion = Convert.ToInt32(versionParts[3], CultureInfo.InvariantCulture);

////            bool isSilverlightPlatform = true;

////#if WPF
////            isSilverlightPlatform = false;
////#endif

			string platform = frameworkVersion > 1000 ? "Silverlight" : "WPF";
			////string platform = isSilverlightPlatform ? "Silverlight" : "WPF";

			string versionNumberDefaultValue = string.Format(
				CultureInfo.InvariantCulture,
				"RadControls for {0} Q{1} {2}, released on {3:dd MMMM yyyy}",
				platform,
				quarter,
				year,
				new DateTime(year, month, day));

			VersionTextProperty = DependencyProperty.Register(
				"VersionText",
				typeof(string),
				typeof(VersionPresenter),
				new PropertyMetadata(versionNumberDefaultValue));

			LoadedExampleProperty = DependencyProperty.Register(
				"LoadedExample",
				typeof(string),
				typeof(VersionPresenter),
				new PropertyMetadata(OnLoadExamplePropertyChanged));
		}

		public VersionPresenter()
		{
			this.DefaultStyleKey = typeof(VersionPresenter);
		}

		private static void OnLoadExamplePropertyChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args)
		{
			VersionPresenter versionPresenter = obj as VersionPresenter;

			//versionPresenter.Visibility = string.Equals(args.NewValue.ToString(), "home", StringComparison.OrdinalIgnoreCase) ?
			//    Visibility.Collapsed : Visibility.Visible;
		}

		public string VersionText
		{
			get { return (string)GetValue(VersionTextProperty); }
			set { SetValue(VersionTextProperty, value); }
		}

		public string LoadedExample
		{
			get { return (string)GetValue(LoadedExampleProperty); }
			set { SetValue(LoadedExampleProperty, value); }
		}
	}
}