﻿using System;
using System.Windows;
using System.Windows.Controls;

namespace Telerik.Windows.QuickStart
{
	public class WatermarkedTextBox : TextBox
	{
		public static readonly DependencyProperty DefaultTextProperty =
			DependencyProperty.Register("DefaultText", typeof(string), typeof(WatermarkedTextBox), new PropertyMetadata("Set Default Text!!!"));

		public string DefaultText
		{
			get { return (string)GetValue(DefaultTextProperty); }
			set { SetValue(DefaultTextProperty, value); }
		}

		public event EventHandler WatermarkedTextChanged = delegate { };

		public WatermarkedTextBox()
		{
			this.TextChanged += this.OnWatermarkedTextBoxTextChanged;
		}

		public override void OnApplyTemplate()
		{
			this.Text = this.DefaultText;

			base.OnApplyTemplate();
		}

		private void OnWatermarkedTextBoxTextChanged(object sender, TextChangedEventArgs e)
		{
			if (!string.Equals(this.Text, this.DefaultText, StringComparison.OrdinalIgnoreCase))
			{
				this.WatermarkedTextChanged(this, e);
			}
		}

		protected override void OnGotFocus(RoutedEventArgs e)
		{
			if (string.Equals(this.Text, this.DefaultText, StringComparison.OrdinalIgnoreCase))
			{
				this.Text = string.Empty;
			}
			base.OnGotFocus(e);
		}

		protected override void OnLostFocus(RoutedEventArgs e)
		{
			if (string.IsNullOrEmpty(this.Text))
			{
				this.Text = this.DefaultText;
			}
			base.OnLostFocus(e);
		}
	}
}
