﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;

namespace Telerik.Windows.QuickStart
{
    /// <summary>
    /// 
    /// </summary>
    public class PageFadeTransition : ITransition
    {
        private object target;
        private TimeSpan timeDuration;
        private Control newPage;
        private Control currentPage;
        private Storyboard currentAnimation;

        /// <summary>
        /// Initializes a new instance of the FadeTransition class.
        /// </summary>
        /// <param name="target">The target.</param>
        /// <param name="duration">The duration.</param>
        public PageFadeTransition(Panel target, TimeSpan duration)
        {
            this.Target = target;
            this.timeDuration = duration;
            this.IsRunning = false;
        }

        /// <summary>
        /// Initializes a new instance of the FadeTransition class.
        /// </summary>
        public PageFadeTransition()
        {
        }

        /// <summary>
        /// Gets or sets the target.
        /// </summary>
        /// <value>The target.</value>
        public object Target
        {
            get
            {
                return this.target;
            }

            set
            {
                if (value is Panel)
                {
                    this.target = value;
                }
                else
                {
                    throw new NotImplementedException();
                }
            }
        }

        /// <summary>
        /// Gets or sets the current story board.
        /// </summary>
        /// <value>The current story board.</value>
        public Storyboard CurrentStoryboard
        {
            get
            {
                return this.currentAnimation;
            }
            set
            {
                this.currentAnimation = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is running.
        /// </summary>
        /// <value>
        /// 	<c>True</c> if this instance is running; otherwise, <c>false</c>.
        /// </value>
        public bool IsRunning
        {
            get;
            set;
        }

		/// <summary>
		/// Stops this instance.
		/// </summary>
		public void StopAnimation()
        {
            if (this.CurrentStoryboard != null)
            {
                this.CurrentStoryboard.Stop();
            }
        }

        /// <summary>
        /// Transitions the page.
        /// </summary>
        /// <param name="previousFrame">The previous frame.</param>
        /// <param name="nextFrame">The next frame.</param>
        public void PerformFrameTransition(IFrame previousFrame, IFrame nextFrame)
        {
            this.IsRunning = true;
            this.newPage = nextFrame as Control;
            this.currentPage = previousFrame as Control;
            if (this.currentPage != this.newPage)
            {
                if (this.currentPage != null)
                {
                    this.currentPage.RenderTransform = null;
                    Duration duration = new Duration(this.timeDuration);
                    DoubleAnimation animation = new DoubleAnimation();
                    animation.Duration = duration;
                    animation.To = 0;
                    Storyboard fadeIn = new Storyboard();
                    this.CurrentStoryboard = fadeIn;
                    fadeIn.Duration = duration;
                    fadeIn.Children.Add(animation);
                    fadeIn.Completed += this.FadeIn_Completed;
                    Storyboard.SetTarget(animation, this.currentPage);
                    Storyboard.SetTargetProperty(animation, new PropertyPath("Opacity"));

                    fadeIn.Begin();
                }
                else
                {
                    Panel panel = ((Panel)this.Target);
                    panel.Children.Clear();
                    panel.Children.Add(this.newPage);
                    panel.UpdateLayout();
                }
            }

            Application targetApp = Application.Current;
            //this.newPage.Height = targetApp.RootVisual.DesiredSize.Height;
        }

        private void FadeIn_Completed(object sender, EventArgs e)
        {
            Storyboard storyBoard = sender as Storyboard;
            storyBoard.Stop();
            Panel panel = ((Panel)this.Target);

            // panel.Children.Remove(this.currentPage);
            panel.Children.Clear();
            this.currentPage = null;

            if (!panel.Children.Contains(this.newPage))
            {
                panel.Children.Add(this.newPage);
                this.newPage.RenderTransform = null;
            }
            else
            {
                RadFrameContainer nextPanel = (this.newPage.Parent as RadFrameContainer);
                if (nextPanel != null)
                {
                    nextPanel.Children.Clear();
                    panel.Children.Add(this.newPage);
                    panel.UpdateLayout();
                }
            }
            this.newPage.RenderTransform = null;
            TimeSpan fadeOutDuration = TimeSpan.FromSeconds(this.timeDuration.TotalSeconds / 2);
            Duration duration = new Duration(fadeOutDuration);
            DoubleAnimation animation = new DoubleAnimation();
            animation.Duration = duration;
            animation.From = 0;
            animation.To = 1;
            Storyboard fadeOut = new Storyboard();
            this.CurrentStoryboard = fadeOut;
            fadeOut.Duration = duration;
            fadeOut.Children.Add(animation);
            Storyboard.SetTarget(animation, this.newPage);
            Storyboard.SetTargetProperty(animation, new PropertyPath("Opacity"));
            fadeOut.Completed += new EventHandler(this.FadeOut_Completed);
            fadeOut.Begin();
        }

        private void FadeOut_Completed(object sender, EventArgs e)
        {
            Storyboard storyBoard = sender as Storyboard;
            storyBoard.Stop();
            this.IsRunning = false;
        }
    }
}
